subroutine eigen_adb_mhd_cons(NX,NVAR,gamma,d,vx,vy,vz,h,&
          Bx,by,bz,Xfac,Yfac,lambda,rem,lem)
!
! MHD isothermal Roe eigenvalues and eigenvectors
!
! Input Arguments:
!   NX    = number of zones in sweep direction
!   NVAR  = number of variables in eigenvalue problem (determined by physics)
!   gamma = adiabatic exponent
!   W     = PRIMATIVE variables at cell centers along 1-D slice
!           W=(d,Vx,Vy,Vz,P,By,Bz)
!   Bx    = magnetic field in sweep direction
!   d     = Roe density
!   vx    = Roe X velocity 
!   vy    = Roe Y velocity 
!   vz    = Roe Z velocity 
!   by    = Roe Y magnetic field
!   bz    = Roe Z magnetic field
!   h     = Roe enthalpy
!   Xfac  = ((by^2-byl*byr)+bz^2-bzl*bzr))/(2*rho)
!   Yfac  = (rho_l+rho_r)/(2*rho)
!
!
! Output Arguments:
!
!   lambda = eigenvalues
!   rem     = right eigenmatrix
!   lem     = left  eigenmatrix
!
! 
! AUTHORS:
!     James M. Stone, Princeton Univeristy
!     Thomas Gardiner, University of Maryland
!     Peter J. Teuben, University of Maryland
!     John Hawley, University of Virginia
!
!     This software was produced with support from NSF Grant AST-0113571 
!
! Copyright 2003 James M. Stone, Thomas Gardiner,
!                Peter Teuben, John F. Hawley
!
!   This program is free software; you can redistribute it and/or modify
!   it under the terms of the GNU General Public License as published by
!   the Free Software Foundation.
!
!   This program is distributed in the hope that it will be useful,
!   but WITHOUT ANY WARRANTY; without even the implied warranty of
!   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!   GNU General Public License for more details.
!

use athenadefs
IMPLICIT NONE

integer, intent(IN) :: NX, NVAR
real (r_kind), intent(IN) :: gamma
real (r_kind), dimension(NX), intent(IN) :: d, vx, vy, vz, h
real (r_kind), dimension(NX), intent(IN) :: Bx, by, bz, Xfac, Yfac
real (r_kind), dimension(NVAR, NX), intent(OUT) :: lambda
real (r_kind), dimension(NVAR, NVAR, NX), intent(OUT) :: rem, lem

!
! LOCALS
!
real (r_kind) :: btsq, bt_starsq, bt, bt_star
real (r_kind) :: vsq, vax,  vaxsq, hp, twid_asq, q_starsq 
real (r_kind) :: cfsq, cfast, cssq, cslow
real (r_kind) :: beta_y, beta_z, beta_ystar, beta_zstar, beta_starsq, vbeta
real (r_kind) :: alpha_f, alpha_s, droot, s, twid_a
real (r_kind) :: Qfast, Qslow, af_prime, as_prime, Afpbb, Aspbb, na
real (r_kind) :: cff, css, af, as, Afpb, Aspb, vqstr, norm
real (r_kind) :: Q_ystar, Q_zstar

integer i
!
!-----------------------------------------------------------------------

EIGEN: do i=1,NX

   vsq = vx(i)**2+vy(i)**2+vz(i)**2
   btsq = by(i)**2+bz(i)**2
   bt_starsq = (gamma-1. - (gamma-2.)*Yfac(i))*btsq
   bt=sqrt(btsq)
   bt_star = sqrt(bt_starsq)

   vaxsq = Bx(i)**2/d(i)
   vax = sqrt(vaxsq)

   hp = h(i) - (vaxsq + btsq/d(i))
   twid_asq = ((gamma-1.)*(hp-.5*vsq)-(gamma-2.)*Xfac(i))
   twid_asq = MAX(twid_asq,TINY_NUMBER)
   q_starsq = twid_asq+(vaxsq+bt_starsq/d(i))

   cfsq = .5*(q_starsq + sqrt(q_starsq**2-4.0*twid_asq*vaxsq))
   cfast = sqrt(cfsq)

   cssq = .5*(q_starsq - sqrt(q_starsq**2-4.0*twid_asq*vaxsq))
   if (cssq .le. 0.) cssq = 0.
   cslow = sqrt(cssq)

   if (bt .eq. 0) then
      beta_y = .5*sqrt(2.)
      beta_z = .5*sqrt(2.)
      beta_ystar = .5*sqrt(2.)
      beta_zstar = .5*sqrt(2.)
   else
      beta_y = by(i)/bt
      beta_z = bz(i)/bt
      beta_ystar = by(i)/bt_star
      beta_zstar = bz(i)/bt_star
   endif    
   beta_starsq = beta_ystar**2 + beta_zstar**2
   vbeta = vy(i)*beta_ystar + vz(i)*beta_zstar

   if ( (cfsq - cssq) .eq. 0.) then
       alpha_f = 1.0
       alpha_s = 0.0
   else if ( (twid_asq - cssq) .le. 0.) then
       alpha_f = 0.0
       alpha_s = 1.0
   else if ( (cfsq - twid_asq) .le. 0.) then
       alpha_f = 1.0
       alpha_s = 0.0
   else
       alpha_f = sqrt((twid_asq-cssq)/(cfsq-cssq))
       alpha_s = sqrt((cfsq-twid_asq)/(cfsq-cssq))
   endif

!
! compute Qs and As for eigenmatrices
!
   droot = sqrt(d(i))
   s  = SIGN(1._r_kind,bx(i))
   twid_a = sqrt(twid_asq)
   Qfast = s*cfast*alpha_f
   Qslow = s*cslow*alpha_s
   af_prime = twid_a*alpha_f/droot
   as_prime = twid_a*alpha_s/droot
   Afpbb = af_prime*bt_star*beta_starsq
   Aspbb = as_prime*bt_star*beta_starsq

!
! eigenvalues
!
!
  lambda(1,i) = vx(i) - cfast
  lambda(2,i) = vx(i) - vax
  lambda(3,i) = vx(i) - cslow
  lambda(4,i) = vx(i)
  lambda(5,i) = vx(i) + cslow
  lambda(6,i) = vx(i) + vax
  lambda(7,i) = vx(i) + cfast

!
! eigenmatrix
!
  rem(1,1,i) = alpha_f
  rem(1,2,i) = alpha_f*(vx(i)-cfast)
  rem(1,3,i) = alpha_f*vy(i) + Qslow*beta_ystar
  rem(1,4,i) = alpha_f*vz(i) + Qslow*beta_zstar
  rem(1,5,i) = alpha_f*(hp-vx(i)*cfast) + Qslow*vbeta+Aspbb
  rem(1,6,i) = as_prime*beta_ystar
  rem(1,7,i) = as_prime*beta_zstar

  rem(2,1,i) = 0.
  rem(2,2,i) = 0.
  rem(2,3,i) = -beta_z
  rem(2,4,i) =  beta_y
  rem(2,5,i) = -(vy(i)*beta_z - vz(i)*beta_y)
  rem(2,6,i) = -s*beta_z/droot
  rem(2,7,i) =  s*beta_y/droot

  rem(3,1,i) = alpha_s
  rem(3,2,i) = alpha_s*(vx(i)-cslow)
  rem(3,3,i) = alpha_s*vy(i) - Qfast*beta_ystar
  rem(3,4,i) = alpha_s*vz(i) - Qfast*beta_zstar
  rem(3,5,i) = alpha_s*(hp - vx(i)*cslow) - Qfast*vbeta - Afpbb
  rem(3,6,i) = -af_prime*beta_ystar
  rem(3,7,i) = -af_prime*beta_zstar

  rem(4,1,i) = 1.0
  rem(4,2,i) = vx(i)
  rem(4,3,i) = vy(i)
  rem(4,4,i) = vz(i)
  rem(4,5,i) = 0.5*vsq + (gamma-2.)*Xfac(i)/(gamma-1.)
  rem(4,6,i) = 0.
  rem(4,7,i) = 0.

  rem(5,1,i) =  alpha_s
  rem(5,2,i) =  alpha_s*(vx(i)+cslow)
  rem(5,3,i) =  alpha_s*vy(i) + Qfast*beta_ystar
  rem(5,4,i) =  alpha_s*vz(i) + Qfast*beta_zstar
  rem(5,5,i) =  alpha_s*(hp+vx(i)*cslow) + Qfast*vbeta - Afpbb
  rem(5,6,i) =  rem(3,6,i)
  rem(5,7,i) =  rem(3,7,i)

  rem(6,1,i) = 0.
  rem(6,2,i) = 0.
  rem(6,3,i) =  beta_z
  rem(6,4,i) = -beta_y
  rem(6,5,i) = -rem(2,5,i)
  rem(6,6,i) =  rem(2,6,i)
  rem(6,7,i) =  rem(2,7,i)

  rem(7,1,i) =  alpha_f
  rem(7,2,i) = alpha_f*(vx(i)+cfast)
  rem(7,3,i) = alpha_f*vy(i) - Qslow*beta_ystar
  rem(7,4,i) = alpha_f*vz(i) - Qslow*beta_zstar
  rem(7,5,i) = alpha_f*(hp + vx(i)*cfast) - Qslow*vbeta + Aspbb
  rem(7,6,i) =  rem(1,6,i)
  rem(7,7,i) =  rem(1,7,i)

! 
! Left eignematrix
!
! normalize some of the quantities by 1/(2a^2)
! some by (gamma-1)/2a^2
!
  na  = 0.5/twid_asq
  cff = na*alpha_f*cfast
  css = na*alpha_s*cslow
  Qfast = Qfast*na
  Qslow = Qslow*na
  af = na*af_prime*d(i)
  as = na*as_prime*d(i)
  Afpb = na*af_prime*bt_star
  Aspb = na*as_prime*bt_star
!
  alpha_f = (gamma-1.)*na*alpha_f
  alpha_s = (gamma-1.)*na*alpha_s
  Q_ystar = beta_ystar/beta_starsq
  Q_zstar = beta_zstar/beta_starsq
  vqstr   = (vy(i)*Q_ystar+vz(i)*Q_zstar)
  norm = (gamma-1.)*2.*na

  lem(1,1,i) = alpha_f*(vsq-hp) + cff*(cfast+vx(i)) - Qslow*vqstr - Aspb
  lem(2,1,i) = -alpha_f*vx(i) - cff
  lem(3,1,i) = -alpha_f*vy(i) + Qslow*Q_ystar
  lem(4,1,i) = -alpha_f*vz(i) + Qslow*Q_zstar
  lem(5,1,i) =  alpha_f
  lem(6,1,i) = as*Q_ystar - alpha_f*by(i)
  lem(7,1,i) = as*Q_zstar - alpha_f*bz(i)

  lem(1,2,i) =  0.5*(vy(i)*beta_z - vz(i)*beta_y)
  lem(2,2,i) =  0.
  lem(3,2,i) = -0.5*beta_z
  lem(4,2,i) =  0.5*beta_y
  lem(5,2,i) =  0.
  lem(6,2,i) = -0.5*droot*beta_z*s
  lem(7,2,i) =  0.5*droot*beta_y*s

  lem(1,3,i) =  alpha_s*(vsq-hp) + css*(cslow+vx(i)) + Qfast*vqstr + Afpb
  lem(2,3,i) = -alpha_s*vx(i) - css
  lem(3,3,i) = -alpha_s*vy(i) - Qfast*Q_ystar
  lem(4,3,i) = -alpha_s*vz(i) - Qfast*Q_zstar
  lem(5,3,i) =  alpha_s
  lem(6,3,i) = -af*Q_ystar - alpha_s*by(i)
  lem(7,3,i) = -af*Q_zstar - alpha_s*bz(i)

  lem(1,4,i) =  1. - norm*(.5*vsq + (gamma-2.)*Xfac(i)/(gamma-1.))
  lem(2,4,i) =  norm*vx(i)
  lem(3,4,i) =  norm*vy(i)
  lem(4,4,i) =  norm*vz(i)
  lem(5,4,i) = -norm
  lem(6,4,i) =  norm*by(i)
  lem(7,4,i) =  norm*bz(i)

  lem(1,5,i) =  alpha_s*(vsq-hp) + css*(cslow-vx(i)) - Qfast*vqstr + Afpb
  lem(2,5,i) = -alpha_s*vx(i) + css 
  lem(3,5,i) = -alpha_s*vy(i) + Qfast*Q_ystar 
  lem(4,5,i) = -alpha_s*vz(i) + Qfast*Q_zstar 
  lem(5,5,i) =  alpha_s
  lem(6,5,i) =  lem(6,3,i)
  lem(7,5,i) =  lem(7,3,i)

  lem(1,6,i) = -lem(1,2,i)
  lem(2,6,i) =  0.
  lem(3,6,i) = -lem(3,2,i)
  lem(4,6,i) = -lem(4,2,i)
  lem(5,6,i) =  0.
  lem(6,6,i) =  lem(6,2,i)
  lem(7,6,i) =  lem(7,2,i)

  lem(1,7,i) =  alpha_f*(vsq-hp) + cff*(cfast-vx(i)) + Qslow*vqstr -Aspb
  lem(2,7,i) = -alpha_f*vx(i) + cff
  lem(3,7,i) = -alpha_f*vy(i) - Qslow*Q_ystar
  lem(4,7,i) = -alpha_f*vz(i) - Qslow*Q_zstar
  lem(5,7,i) =  alpha_f
  lem(6,7,i) =  lem(6,1,i)
  lem(7,7,i) =  lem(7,1,i)

end do EIGEN


return


end subroutine eigen_adb_mhd_cons
