subroutine eigen_adb_mhd_prim(NX,NVAR,gamma,W,bx,lambda,rem,lem)
!
! MHD adiabatic Roe eigenvalues and eigenvectors
! primitive variables
! adiabatic MHD
!
! Input Arguments:
!   NX    = number of zones in sweep direction
!   NVAR  = number of variables in eigenvalue problem (determined by physics)
!   gamma = adiabatic exponent
!   W     = PRIMATIVE variables at cell centers along 1-D slice
!           W=(d,Vx,Vy,Vz,P,By,Bz)
!   Bx    = magnetic field in sweep direction
!
! Output Arguments:
!
!   lambda = eigenvalues
!   rem     = right eigenmatrix
!   lem     = left  eigenmatrix
!
! 
! AUTHORS:
!     James M. Stone, Princeton Univeristy
!     Thomas Gardiner, University of Maryland
!     Peter J. Teuben, University of Maryland
!     John Hawley, University of Virginia
!
!     This software was produced with support from NSF Grant AST-0113571 
!
! Copyright 2003 James M. Stone, Thomas Gardiner,
!                Peter Teuben, John F. Hawley
!
!   This program is free software; you can redistribute it and/or modify
!   it under the terms of the GNU General Public License as published by
!   the Free Software Foundation.
!
!   This program is distributed in the hope that it will be useful,
!   but WITHOUT ANY WARRANTY; without even the implied warranty of
!   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!   GNU General Public License for more details.
!

use athenadefs
IMPLICIT NONE

integer, intent(IN) :: NX, NVAR
real (r_kind), intent(IN) :: gamma
real (r_kind), dimension(NX), intent(IN) :: bx
real (r_kind), dimension(NX,NVAR), target, intent(IN) :: W

real (r_kind), intent(OUT), dimension(NVAR,NX) :: lambda
real (r_kind), intent(OUT), dimension(NVAR,NVAR,NX) :: rem, lem
!
!
! LOCALS

real (r_kind)  :: cs, vax, cfast, cslow, droot, na, s
real (r_kind)  :: beta_y, beta_z, alpha_f, alpha_s
real (r_kind), dimension(:), pointer :: d, vx, p, by, bz

integer i

!
!-----------------------------------------------------------------------

d  => w(:,1)
vx => w(:,2)
p  => w(:,5)
by => w(:,6)
bz => w(:,7)


!
! Compute various wave speeds
!
EIGEN: do i=1,NX
  cs = sqrt(gamma*p(i)/d(i))
  vax = sqrt(bx(i)*bx(i)/d(i))
  cfast = 0.5_r_kind * ( cs*cs+vax*vax+(by(i)*by(i)+bz(i)*bz(i))/d(i) &
        + sqrt((cs*cs+vax*vax+(by(i)*by(i)+bz(i)*bz(i))/d(i))**2-(2._r_kind*cs*vax)**2) )
  cfast = sqrt(cfast)
  cslow = 0.5_r_kind * ( cs*cs+vax*vax+(by(i)*by(i)+bz(i)*bz(i))/d(i) &
        - sqrt((cs*cs+vax*vax+(by(i)*by(i)+bz(i)*bz(i))/d(i))**2-(2._r_kind*cs*vax)**2) )
  cslow = MAX(cslow,0._r_kind)
  cslow = sqrt(cslow)
  droot = sqrt(d(i))
  na    = 1./(2.*cs*cs)

  if ( (by(i)*by(i)+bz(i)*bz(i)) .gt. 0.) then
    beta_y =  by(i)/(sqrt(by(i)*by(i)+bz(i)*bz(i))) 
    beta_z =  bz(i)/(sqrt(by(i)*by(i)+bz(i)*bz(i)))
  else
    beta_y = .5*sqrt(2._r_kind) 
    beta_z = .5*sqrt(2._r_kind)
  endif

    if ( (cfast**2 - cslow**2) .eq. 0.) then
       alpha_f = 1.0
       alpha_s = 0.0
    else if ( (cs**2 - cslow**2) .le. 0.) then
       alpha_f = 0.0
       alpha_s = 1.0
    else if ( (cfast**2 - cs**2) .le. 0.) then
       alpha_f = 1.0
       alpha_s = 0.0
    else
       alpha_f = sqrt((cs**2-cslow**2)/(cfast**2-cslow**2))
       alpha_s = sqrt((cfast**2-cs**2)/(cfast**2-cslow**2))
    endif

    s = SIGN(1._r_kind,bx(i))
!
! eigenvalues
!
!
  lambda(1,i) = vx(i) - cfast
  lambda(2,i) = vx(i) - vax
  lambda(3,i) = vx(i) - cslow
  lambda(4,i) = vx(i)
  lambda(5,i) = vx(i) + cslow
  lambda(6,i) = vx(i) + vax
  lambda(7,i) = vx(i) + cfast
!
! eigenmatrix
! set everything to zero and then
! just reset the specific nonzero values
!
  rem(1,1,i) = d(i)*alpha_f
  rem(1,2,i) = -cfast*alpha_f
  rem(1,3,i) = s*cslow*alpha_s * beta_y
  rem(1,4,i) = s*cslow*alpha_s * beta_z
  rem(1,5,i) = d(i)*cs**2*alpha_f
  rem(1,6,i) = cs*alpha_s*droot*beta_y
  rem(1,7,i) = cs*alpha_s*droot*beta_z

  rem(2,1,i) = 0.
  rem(2,2,i) = 0.
  rem(2,3,i) = -beta_z
  rem(2,4,i) =  beta_y
  rem(2,5,i) = 0.
  rem(2,6,i) = -s*beta_z*droot
  rem(2,7,i) =  s*beta_y*droot

  rem(3,1,i) = d(i)*alpha_s
  rem(3,2,i) = -cslow*alpha_s
  rem(3,3,i) = -s*cfast*alpha_f*beta_y
  rem(3,4,i) = -s*cfast*alpha_f*beta_z
  rem(3,5,i) = d(i)*cs**2*alpha_s
  rem(3,6,i) = -s*cs*alpha_f*droot*beta_y
  rem(3,7,i) = -s*cs*alpha_f*droot*beta_z

  rem(4,1,i) = 1.0
  rem(4,2,i) = 0.
  rem(4,3,i) = 0.
  rem(4,4,i) = 0.
  rem(4,5,i) = 0.
  rem(4,6,i) = 0.
  rem(4,7,i) = 0.

  rem(5,1,i) =  rem(3,1,i)
  rem(5,2,i) = -rem(3,2,i)
  rem(5,3,i) = -rem(3,3,i)
  rem(5,4,i) = -rem(3,4,i)
  rem(5,5,i) =  rem(3,5,i)
  rem(5,6,i) =  rem(3,6,i)
  rem(5,7,i) =  rem(3,7,i)

  rem(6,1,i) = 0.
  rem(6,2,i) = 0.
  rem(6,3,i) =  beta_z
  rem(6,4,i) = -beta_y
  rem(6,5,i) = 0.
  rem(6,6,i) = rem(2,6,i)
  rem(6,7,i) = rem(2,7,i)

  rem(7,1,i) =  rem(1,1,i)
  rem(7,2,i) = -rem(1,2,i)
  rem(7,3,i) = -rem(1,3,i)
  rem(7,4,i) = -rem(1,4,i)
  rem(7,5,i) =  rem(1,5,i)
  rem(7,6,i) =  rem(1,6,i)
  rem(7,7,i) =  rem(1,7,i)


  lem(1,1,i) = 0.
  lem(2,1,i) = -na*cfast*alpha_f
  lem(3,1,i) = na*rem(1,3,i)
  lem(4,1,i) = na*rem(1,4,i)
  lem(5,1,i) = na*alpha_f/d(i)
  lem(6,1,i) = na*s*cs*alpha_s/droot*beta_y
  lem(7,1,i) = na*s*cs*alpha_s/droot*beta_z

  lem(1,2,i) =  0.
  lem(2,2,i) =  0.
  lem(3,2,i) = -0.5*beta_z
  lem(4,2,i) =  0.5*beta_y
  lem(5,2,i) =  0.
  lem(6,2,i) = -0.5*beta_z*s/droot
  lem(7,2,i) =  0.5*beta_y*s/droot

  lem(1,3,i) =  0.
  lem(2,3,i) = -na*cslow*alpha_s
  lem(3,3,i) = -na*s*cfast*alpha_f*beta_y
  lem(4,3,i) = -na*s*cfast*alpha_f*beta_z
  lem(5,3,i) =  na*alpha_s/d(i)
  lem(6,3,i) = -na*s*cs*alpha_f/droot*beta_y
  lem(7,3,i) = -na*s*cs*alpha_f/droot*beta_z

  lem(1,4,i) =  1.
  lem(2,4,i) =  0.
  lem(3,4,i) =  0.
  lem(4,4,i) =  0.
  lem(5,4,i) = -1./(cs*cs)
  lem(6,4,i) =  0.
  lem(7,4,i) =  0.

  lem(1,5,i) =  0.
  lem(2,5,i) =  -lem(2,3,i)
  lem(3,5,i) =  -lem(3,3,i)
  lem(4,5,i) =  -lem(4,3,i)
  lem(5,5,i) =   lem(5,3,i)
  lem(6,5,i) =   lem(6,3,i)
  lem(7,5,i) =   lem(7,3,i)

  lem(1,6,i) =  0.
  lem(2,6,i) =  0.
  lem(3,6,i) = -lem(3,2,i)
  lem(4,6,i) = -lem(4,2,i)
  lem(5,6,i) =  0.
  lem(6,6,i) =  lem(6,2,i)
  lem(7,6,i) =  lem(7,2,i)

  lem(1,7,i) =  0.
  lem(2,7,i) = -lem(2,1,i)
  lem(3,7,i) = -lem(3,1,i)
  lem(4,7,i) = -lem(4,1,i)
  lem(5,7,i) =  lem(5,1,i)
  lem(6,7,i) =  lem(6,1,i)
  lem(7,7,i) =  lem(7,1,i)

end do EIGEN

NULLIFY(d,vx,p,by,bz)

return

end subroutine eigen_adb_mhd_prim
