subroutine eigen_iso_mhd_cons(NX,NVAR,Isoc,d,vx,vy,vz,&
          Bx,by,bz,Xfac,Yfac,lambda,rem,lem)
!
! MHD isothermal Roe eigenvalues and eigenvectors
!
! Input Arguments:
!   NX    = number of zones in sweep direction
!   NVAR  = number of variables in eigenvalue problem (determined by physics)
!   Isoc  = isothermal sound speed
!   W     = PRIMATIVE variables at cell centers along 1-D slice
!           W=(d,Vx,Vy,Vz,By,Bz)
!   Bx    = magnetic field in sweep direction
!   d     = Roe density
!   vx    = Roe X velocity
!   vy    = Roe Y velocity
!   vz    = Roe Z velocity
!   by    = Roe Y magnetic field
!   bz    = Roe Z magnetic field
!   Xfac  = ((by^2-byl*byr)+bz^2-bzl*bzr))/(2*rho)
!   Yfac  = (rho_l+rho_r)/(2*rho)
!
!
! Output Arguments:
!
!   lambda = eigenvalues
!   rem     = right eigenmatrix
!   lem     = left  eigenmatrix
!
! AUTHORS:
!     James M. Stone, Princeton Univeristy
!     Thomas Gardiner, University of Maryland
!     Peter J. Teuben, University of Maryland
!     John Hawley, University of Virginia
!
!     This software was produced with support from NSF Grant AST-0113571 
!
! Copyright 2003 James M. Stone, Thomas Gardiner,
!                Peter Teuben, John F. Hawley
!
!   This program is free software; you can redistribute it and/or modify
!   it under the terms of the GNU General Public License as published by
!   the Free Software Foundation.
!
!   This program is distributed in the hope that it will be useful,
!   but WITHOUT ANY WARRANTY; without even the implied warranty of
!   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!   GNU General Public License for more details.
!

use athenadefs
IMPLICIT NONE

integer, intent(IN) :: NX, NVAR
real (r_kind), intent(IN) :: Isoc
real (r_kind), dimension(NX), intent(IN) :: d, vx, vy, vz
real (r_kind), dimension(NX), intent(IN) :: Bx, by, bz, Xfac, Yfac
real (r_kind), dimension(NVAR, NX), intent(OUT) :: lambda
real (r_kind), dimension(NVAR, NVAR, NX), intent(OUT) :: rem, lem

!
! LOCALS
!
real (r_kind) :: btsq, bt_starsq, bt, bt_star
real (r_kind) :: vax,  vaxsq, twid_csq, q_starsq
real (r_kind) :: cfsq, cf, cssq, cs
real (r_kind) :: bet2, bet3, bet2_star, bet3_star, bet_starsq
real (r_kind) :: alpha_f, alpha_s, sqrtd, s, twid_c
real (r_kind) :: Qf, Qs, af_prime, as_prime, Afpbb, Aspbb, na
real (r_kind) :: cff, css, af, as, Afpb, Aspb, vqstr, norm
real (r_kind) :: Q2_star, Q3_star



integer i
!
!-----------------------------------------------------------------------


EIGEN: do i=1,NX

  btsq = by(i)**2 + bz(i)**2
  bt_starsq = btsq*Yfac(i)

! Compute fast- and slow-magnetosonic speeds (eq. B42 and B43) 

   vaxsq = bx(i)**2/d(i)
   vax   = sqrt(vaxsq)
   twid_csq = Isoc**2 + Xfac(i)
   q_starsq  = twid_csq + (vaxsq + bt_starsq/d(i))
   cfsq = 0.5*(q_starsq + sqrt(q_starsq*q_starsq - 4.0*twid_csq*vaxsq))
   cf  = sqrt(cfsq)
   cssq = 0.5*(q_starsq - sqrt(q_starsq*q_starsq - 4.0*twid_csq*vaxsq))
   if (cssq .LE. 0.0) cssq = 0.0
   cs  = sqrt(cssq)

! Compute beta's (eqs A13, B29) 

   bt = sqrt(btsq)
   bt_star = sqrt(bt_starsq)
   if (bt .EQ. 0.0) then
      bet2 = 1./sqrt(2._r_kind)
      bet3 = 1./sqrt(2._r_kind)
      bet2_star = 1./sqrt(2._r_kind)
      bet3_star = 1./sqrt(2._r_kind)
   else 
      bet2 = by(i)/bt
      bet3 = bz(i)/bt
      bet2_star = by(i)/bt_star
      bet3_star = bz(i)/bt_star
   endif
   bet_starsq = bet2_star*bet2_star + bet3_star*bet3_star

! Compute alpha's (eq A12) 

   if ((cfsq-cssq) .EQ. 0.0) then
      alpha_f = 1.0
      alpha_s = 0.0
   else if ((twid_csq - cssq) .LE. 0.0) then
      alpha_f = 0.0
      alpha_s = 1.0
   else if ((cfsq - twid_csq) .LE. 0.0) then
      alpha_f = 1.0
      alpha_s = 0.0
   else 
      alpha_f = sqrt((twid_csq - cssq)/(cfsq - cssq))
      alpha_s = sqrt((cfsq - twid_csq)/(cfsq - cssq))
   end if

! Compute Q's (eq. A11), etc. 

   sqrtd = sqrt(d(i))
   s = SIGN(1._r_kind,bx(i))
   twid_c = sqrt(twid_csq)
   qf = cf*alpha_f*s
   qs = cs*alpha_s*s
   af_prime = twid_c*alpha_f/sqrtd
   as_prime = twid_c*alpha_s/sqrtd

!
! eigenvalues
!
  lambda(1,i) = vx(i) - cf
  lambda(2,i) = vx(i) - vax
  lambda(3,i) = vx(i) - cs
  lambda(4,i) = vx(i) + cs
  lambda(5,i) = vx(i) + vax
  lambda(6,i) = vx(i) + cf

!
! right eigenmatrix
!
  rem(1,1,i) = alpha_f
  rem(1,2,i) = alpha_f*(vx(i)-cf)
  rem(1,3,i) = alpha_f*vy(i) + qs*bet2_star
  rem(1,4,i) = alpha_f*vz(i) + qs*bet3_star
  rem(1,5,i) = as_prime*bet2_star
  rem(1,6,i) = as_prime*bet3_star

  rem(2,1,i) = 0.
  rem(2,2,i) = 0.
  rem(2,3,i) = -bet3
  rem(2,4,i) =  bet2
  rem(2,5,i) = -bet3*s/sqrtd
  rem(2,6,i) =  bet2*s/sqrtd

  rem(3,1,i) = alpha_s
  rem(3,2,i) = alpha_s*(vx(i)-cs)
  rem(3,3,i) = alpha_s*vy(i) - qf*bet2_star
  rem(3,4,i) = alpha_s*vz(i) - qf*bet3_star
  rem(3,5,i) = -af_prime*bet2_star
  rem(3,6,i) = -af_prime*bet3_star

  rem(4,1,i) =  alpha_s
  rem(4,2,i) =  alpha_s*(vx(i)+cs)
  rem(4,3,i) =  alpha_s*vy(i) + qf*bet2_star
  rem(4,4,i) =  alpha_s*vz(i) + qf*bet3_star
  rem(4,5,i) =  rem(3,5,i)
  rem(4,6,i) =  rem(3,6,i)

  rem(5,1,i) = 0.
  rem(5,2,i) = 0.
  rem(5,3,i) =  bet3
  rem(5,4,i) = -bet2
  rem(5,5,i) =  rem(2,5,i)
  rem(5,6,i) =  rem(2,6,i)

  rem(6,1,i) = alpha_f
  rem(6,2,i) = alpha_f*(vx(i)+cf)
  rem(6,3,i) = alpha_f*vy(i) - qs*bet2_star
  rem(6,4,i) = alpha_f*vz(i) - qs*bet3_star
  rem(6,5,i) = rem(1,5,i)
  rem(6,6,i) = rem(1,6,i)

! left eigenmatrix

   norm = 0.5/twid_csq
   cff = norm*alpha_f*cf
   css = norm*alpha_s*cs
   qf  = qf*norm
   qs  = qs*norm
   af = norm*af_prime*d(i)
   as = norm*as_prime*d(i)
   afpb = norm*af_prime*bt_star
   aspb = norm*as_prime*bt_star

   q2_star = bet2_star/bet_starsq
   q3_star = bet3_star/bet_starsq
   vqstr = (vy(i)*q2_star + vz(i)*q3_star)

  lem(1,1,i) =  cff*(cf+vx(i)) - qs*vqstr - Aspb
  lem(2,1,i) = -cff
  lem(3,1,i) =  Qs*Q2_star
  lem(4,1,i) =  Qs*Q3_star
  lem(5,1,i) = as*Q2_star
  lem(6,1,i) = as*Q3_star

  lem(1,2,i) =  0.5*(vy(i)*bet3 - vz(i)*bet2)
  lem(2,2,i) =  0.
  lem(3,2,i) = -0.5*bet3
  lem(4,2,i) =  0.5*bet2
  lem(5,2,i) = -0.5*sqrtd*bet3*s
  lem(6,2,i) =  0.5*sqrtd*bet2*s

  lem(1,3,i) =  css*(cs+vx(i)) + Qf*vqstr + Afpb
  lem(2,3,i) = -css
  lem(3,3,i) = -Qf*Q2_star
  lem(4,3,i) = -Qf*Q3_star
  lem(5,3,i) = -af*Q2_star
  lem(6,3,i) = -af*Q3_star

  lem(1,4,i) =  css*(cs-vx(i)) - Qf*vqstr + Afpb
  lem(2,4,i) =  css
  lem(3,4,i) = -lem(3,3,i)
  lem(4,4,i) = -lem(4,3,i)
  lem(5,4,i) =  lem(5,3,i)
  lem(6,4,i) =  lem(6,3,i)

  lem(1,5,i) = -lem(1,2,i)
  lem(2,5,i) =  0.
  lem(3,5,i) = -lem(3,2,i)
  lem(4,5,i) = -lem(4,2,i)
  lem(5,5,i) =  lem(5,2,i)
  lem(6,5,i) =  lem(6,2,i)

  lem(1,6,i) =  cff*(cf-vx(i)) + Qs*vqstr -Aspb
  lem(2,6,i) =  cff
  lem(3,6,i) = -lem(3,1,i)
  lem(4,6,i) = -lem(4,1,i)
  lem(5,6,i) =  lem(5,1,i)
  lem(6,6,i) =  lem(6,1,i)

end do EIGEN
end subroutine eigen_iso_mhd_cons
