subroutine eigen_iso_mhd_prim(NX,NVAR,isoc,W,bx,lambda,rem,lem)
!
! MHD isothermal Roe eigenvalues and eigenvectors
! primitive variables
! Isothermal MHD
!
! Input Arguments:
!   NX    = number of zones in sweep direction
!   NVAR  = number of variables in eigenvalue problem (determined by physics)
!   isoc  = isothermal sound speed
!   W     = PRIMATIVE variables at cell centers along 1-D slice
!           W=(d,Vx,Vy,Vz,By,Bz)
!   Bx    = magnetic field in sweep direction
!
! Output Arguments:
!
!   lambda = eigenvalues
!   rem     = right eigenmatrix
!   lem     = left  eigenmatrix
!
! AUTHORS:
!     James M. Stone, Princeton Univeristy
!     Thomas Gardiner, University of Maryland
!     Peter J. Teuben, University of Maryland
!     John Hawley, University of Virginia
!
!     This software was produced with support from NSF Grant AST-0113571 
!
! Copyright 2003 James M. Stone, Thomas Gardiner,
!                Peter Teuben, John F. Hawley
!
!   This program is free software; you can redistribute it and/or modify
!   it under the terms of the GNU General Public License as published by
!   the Free Software Foundation.
!
!   This program is distributed in the hope that it will be useful,
!   but WITHOUT ANY WARRANTY; without even the implied warranty of
!   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!   GNU General Public License for more details.
!

use athenadefs
IMPLICIT NONE

integer, intent(IN) :: NX, NVAR
real (r_kind), intent(IN) :: isoc
real (r_kind), dimension(NX), intent(IN) :: bx
real (r_kind), dimension(NX,NVAR), target, intent(IN) :: W

real (r_kind), intent(OUT), dimension(NVAR,NX) :: lambda
real (r_kind), intent(OUT), dimension(NVAR,NVAR,NX) :: rem, lem
!
! LOCALS

real (r_kind)  :: btsq, vaxsq, q_starsq, cfsq, cf, cssq, cs, bt
real (r_kind)  :: s, qf, qs, af, as, vax, norm, af_prime, as_prime
real (r_kind)  :: bet2, bet3, alpha_f, alpha_s, sqrtd
real (r_kind), dimension(:), pointer :: d, vx, by, bz

integer i

!
!-----------------------------------------------------------------------


d  => w(:,1)
vx => w(:,2)
by => w(:,5)
bz => w(:,6)
 

EIGEN: do i=1,NX

! Compute fast- and slow-magnetosonic speeds (eq. A8 and A9) */
!
   btsq  = by(i)**2 + bz(i)**2
   vaxsq = bx(i)**2/d(i)
   vax = sqrt(vaxsq)
   q_starsq  = (isoc**2) + vaxsq + btsq/d(i)
   cfsq = 0.5*(q_starsq + sqrt(q_starsq*q_starsq-4.*(Isoc**2)*vaxsq))
   cf  = sqrt(cfsq)
   cssq = 0.5*(q_starsq - sqrt(q_starsq*q_starsq-4.*(Isoc**2)*vaxsq))
   if (cssq .LE. 0.0) cssq = 0.0
   cs  = sqrt(cssq)

! Compute beta(s) (eq A13) 

   bt  = sqrt(btsq)
   if (bt .EQ. 0.0) then
      bet2 = 1./sqrt(2._r_kind)
      bet3 = 1./sqrt(2._r_kind)
   else 
      bet2 = by(i)/bt
      bet3 = bz(i)/bt
   endif

! Compute alpha(s) (eq A12) 

   if ((cfsq-cssq) .EQ. 0.0) then
      alpha_f = 1.0
      alpha_s = 0.0
   else if ( (Isoc**2 - cssq) .LE. 0.0) then
      alpha_f = 0.0
      alpha_s = 1.0
   else if ( (cfsq - Isoc**2) .LE. 0.0) then
      alpha_f = 1.0
      alpha_s = 0.0
   else 
      alpha_f = sqrt((Isoc**2 - cssq)/(cfsq - cssq))
      alpha_s = sqrt((cfsq - Isoc**2)/(cfsq - cssq))
   endif

! Compute Q(s) and A(s) (eq. A11), etc. 

   sqrtd = sqrt(d(i))
   s = SIGN(1._r_kind, bx(i))
   qf = cf*alpha_f*s
   qs = cs*alpha_s*s
   af = Isoc*alpha_f*sqrtd
   as = Isoc*alpha_s*sqrtd

! Compute eigenvalues (eq. A17) */

   lambda(1,i) = vx(i) - cf
   lambda(2,i) = vx(i) - vax
   lambda(3,i) = vx(i) - cs
   lambda(4,i) = vx(i) + cs
   lambda(5,i) = vx(i) + vax
   lambda(6,i) = vx(i) + cf

! right eigenmatrix

  rem(1,1,i) = d(i)*alpha_f
  rem(1,2,i) = -cf*alpha_f
  rem(1,3,i) = qs*bet2
  rem(1,4,i) = qs*bet3
  rem(1,5,i) = as*bet2
  rem(1,6,i) = as*bet3

  rem(2,1,i) = 0.
  rem(2,2,i) = 0.
  rem(2,3,i) = -bet3
  rem(2,4,i) =  bet2
  rem(2,5,i) = -bet3*s*sqrtd
  rem(2,6,i) =  bet2*s*sqrtd

  rem(3,1,i) = d(i)*alpha_s
  rem(3,2,i) = -cs*alpha_s
  rem(3,3,i) = -qf*bet2
  rem(3,4,i) = -qf*bet3
  rem(3,5,i) = -af*bet2
  rem(3,6,i) = -af*bet3

  rem(4,1,i) =  rem(3,1,i)
  rem(4,2,i) = -rem(3,2,i)
  rem(4,3,i) = -rem(3,3,i)
  rem(4,4,i) = -rem(3,4,i)
  rem(4,5,i) =  rem(3,5,i)
  rem(4,6,i) =  rem(3,6,i)

  rem(5,1,i) =  0.0
  rem(5,2,i) =  0.0
  rem(5,3,i) =  bet3
  rem(5,4,i) = -bet2
  rem(5,5,i) =  rem(2,5,i)
  rem(5,6,i) =  rem(2,6,i)

  rem(6,1,i) =  rem(1,1,i)
  rem(6,2,i) = -rem(1,2,i)
  rem(6,3,i) = -rem(1,3,i)
  rem(6,4,i) = -rem(1,4,i)
  rem(6,5,i) =  rem(1,5,i)
  rem(6,6,i) =  rem(1,6,i)

!
! left eigenmatrix
!

   norm = 0.5/Isoc**2
   qf = norm*qf
   qs = norm*qs
   af_prime = norm*af/d(i)
   as_prime = norm*as/d(i)

  lem(1,1,i) = norm*alpha_f*isoc**2/d(i)
  lem(2,1,i) = -norm*cf*alpha_f
  lem(3,1,i) =  qs*bet2
  lem(4,1,i) =  qs*bet3
  lem(5,1,i) =  as_prime*bet2
  lem(6,1,i) =  as_prime*bet3

  lem(1,2,i) =  0.
  lem(2,2,i) =  0.
  lem(3,2,i) = -0.5*bet3
  lem(4,2,i) =  0.5*bet2
  lem(5,2,i) = -0.5*bet3*s/sqrtd
  lem(6,2,i) = -0.5*bet2*s/sqrtd

  lem(1,3,i) = norm*alpha_s*isoc**2/d(i) 
  lem(2,3,i) = -norm*cs*alpha_s
  lem(3,3,i) = -qf*bet2
  lem(4,3,i) = -qf*bet3
  lem(5,3,i) = -af_prime*bet2
  lem(6,3,i) = -af_prime*bet3

  lem(1,4,i) =  lem(1,3,i)
  lem(2,4,i) = -lem(2,3,i)
  lem(3,4,i) = -lem(3,3,i)
  lem(4,4,i) = -lem(4,3,i)
  lem(5,4,i) =  lem(5,3,i)
  lem(6,4,i) =  lem(6,3,i)

  lem(1,5,i) =  0.
  lem(2,5,i) =  0.
  lem(3,5,i) =  -lem(3,2,i)
  lem(4,5,i) =  -lem(4,2,i)
  lem(5,5,i) =   lem(5,2,i)
  lem(6,5,i) =   lem(6,2,i)

  lem(1,6,i) =  0.
  lem(2,6,i) = -lem(2,1,i)
  lem(3,6,i) = -lem(3,1,i)
  lem(4,6,i) = -lem(4,1,i)
  lem(5,6,i) =  lem(5,1,i)
  lem(6,6,i) =  lem(6,1,i)

end do EIGEN

NULLIFY(d,vx,by,bz)

end subroutine eigen_iso_mhd_prim
