subroutine flux_roe(NVAR,NX,gamma,Bx,Wleft,Wright,flux,maxev)
! 
! AUTHORS:
!     James M. Stone, Princeton Univeristy
!     Thomas Gardiner, University of Maryland
!     Peter J. Teuben, University of Maryland
!     John Hawley, University of Virginia
!
!     This software was produced with support from NSF Grant AST-0113571 
!
! Copyright 2003 James M. Stone, Thomas Gardiner,
!                Peter Teuben, John F. Hawley
!
!   This program is free software; you can redistribute it and/or modify
!   it under the terms of the GNU General Public License as published by
!   the Free Software Foundation.
!
!   This program is distributed in the hope that it will be useful,
!   but WITHOUT ANY WARRANTY; without even the implied warranty of
!   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!   GNU General Public License for more details.
!
!
! Input Arguments:
!   NX = number of zones in sweep direction
!   NVAR  = number of variables in eigenvalue problem (determined by physics)
!   gamma = adiabatic exponent
!   Bx  = magnetic field in sweep direction
!   Wleft, Wright =  left and right states of primitive variables
!
! Output Arguments
!   flux =  output conservative fluxes
!   maxev = maximum wavespeed eigenvalue for use in timestep calculation
!

use athenadefs
IMPLICIT NONE


integer, intent(IN) ::  NVAR, NX
real (r_kind), intent(IN) :: gamma
real (r_kind), dimension(NX), intent(IN) :: Bx
real (r_kind), dimension(NX,NVAR), target, intent(IN) :: Wleft, Wright
real (r_kind), dimension(NX,NVAR), target, intent(out) :: flux
real (r_kind), intent(OUT) :: maxev

!
! LOCALS
! Variables that are always used for all physics are automatic
! Variables that aren't necessarily used are allocatable
!

real (r_kind), dimension(NVAR,NVAR,NX) :: lem, rem
real (r_kind), dimension(NVAR,NX) :: lambda

real (r_kind), dimension(NX) :: droe, vxroe, vyroe, vzroe, sqrtdl, sqrtdr
real (r_kind), dimension(NX,NVAR), target :: Ul, Ur
real (r_kind), dimension(NX,NVAR) :: fxl, fxr, a
real (r_kind), dimension(NVAR) :: Uim

real (r_kind), dimension(:), allocatable :: hroe, byroe, bzroe, Xfactor, Yfactor
real (r_kind), dimension(:), allocatable :: pbr, pbl

logical, dimension(NX) :: HLLE

!
! Pointers with individual physical variable names are defined for clarity 
! 'l' means left, 'r' means right corresponding to the value to 
! the left or right of the zone interface.
!

real (r_kind), dimension(:), pointer :: fluxd, fluxm1, fluxm2, fluxm3, fluxe
real (r_kind), dimension(:), pointer :: fluxby, fluxbz, byl, byr, bzl, bzr
real (r_kind), dimension(:), pointer :: dl, vxl, vyl, vzl, pl
real (r_kind), dimension(:), pointer :: dr, vxr, vyr, vzr, pr
real (r_kind), dimension(:), pointer :: cdl, mxl, myl, mzl, el
real (r_kind), dimension(:), pointer :: cdr, mxr, myr, mzr, er

real (r_kind) :: csl, csr, bp, bm, vaxsq, ct2, cfsq, cfl, cfr
real (r_kind) :: Pim


integer :: i, n

!----------------------------------------------------------------------------
!
! allocate necessary work arrays if needed for the physics
!

if (adiabatic) allocate(hroe(NX))

if (mhd) then
  allocate(pbl(NX))
  allocate(pbr(NX))
  allocate(byroe(NX))
  allocate(bzroe(NX))
  allocate(Xfactor(NX))
  allocate(Yfactor(NX))
endif

!
! Assign pointers for improved readability
! Fluxes
! left and right primitive variables
! left and right conserved variables
!

fluxd=>flux(:,1); fluxm1=>flux(:,2); fluxm2=>flux(:,3); fluxm3=>flux(:,4)


dl=> Wleft(:,1); vxl=> Wleft(:,2); vyl=> Wleft(:,3); vzl=> Wleft(:,4)
dr=>Wright(:,1); vxr=>Wright(:,2); vyr=>Wright(:,3); vzr=>Wright(:,4)


cdl=>Ul(:,1); mxl=>Ul(:,2); myl=>Ul(:,3); mzl=>Ul(:,4)
cdr=>Ur(:,1); mxr=>Ur(:,2); myr=>Ur(:,3); mzr=>Ur(:,4)

! Assign physics specific terms

SELECT CASE (NVAR)
  CASE(7)
    fluxe  => flux(:,5)
    fluxby => flux(:,6)
    fluxbz => flux(:,7)
    pl     => Wleft(:,5)
    byl    => Wleft(:,6)
    bzl    => Wleft(:,7)
    pr     => Wright(:,5)
    byr    => Wright(:,6)
    bzr    => Wright(:,7)
    el     => Ul(:,5)
    er     => Ur(:,5)
    pbl    = 0.5*(Bx*Bx + byl*byl + bzl*bzl)
    pbr    = 0.5*(Bx*Bx + byr*byr + bzr*bzr)
  CASE(6)
    fluxby => flux(:,5)
    fluxbz => flux(:,6)
    byl    => Wleft(:,5)
    bzl    => Wleft(:,6)
    byr    => Wright(:,5)
    bzr    => Wright(:,6)
    pbl    = 0.5*(Bx*Bx + byl*byl + bzl*bzl)
    pbr    = 0.5*(Bx*Bx + byr*byr + bzr*bzr)

  CASE(5)
    fluxe  => flux(:,5)
    pl     => Wleft(:,5)
    pr     => Wright(:,5)
    el     => Ul(:,5)
    er     => Ur(:,5)

  CASE(4)

  CASE DEFAULT
    write(*,*) 'Incorrect NVAR value in flux_roe: ', NVAR
    stop

END SELECT

!
! step 1:  obtain left and right conserved states from primitive variables
!  The left and right primative values are input variables to this routine
!

Ul(:,1) = dl
Ul(:,2) = dl*vxl
Ul(:,3) = dl*vyl
Ul(:,4) = dl*vzl

Ur(:,1) = dr
Ur(:,2) = dr*vxr
Ur(:,3) = dr*vyr
Ur(:,4) = dr*vzr

SELECT CASE (NVAR)
  CASE (7)
    Ul(:,6) = byl
    Ul(:,7) = bzl
    Ul(:,5) = pl/(gamma-1.) + pbl +.5_r_kind*dl*(vxl**2+vyl**2+vzl**2)
    Ur(:,6) = byr
    Ur(:,7) = bzr
    Ur(:,5) = pr/(gamma-1.) + pbr +.5_r_kind*dr*(vxr**2+vyr**2+vzr**2)
  CASE (6)
    Ul(:,5) = byl
    Ul(:,6) = bzl
    Ur(:,5) = byr
    Ur(:,6) = bzr

  CASE (5)
    el  = pl/(gamma-1.)+.5_r_kind*dl*(vxl**2+vyl**2+vzl**2)
    er  = pr/(gamma-1.)+.5_r_kind*dr*(vxr**2+vyr**2+vzr**2)

  CASE (4)

  CASE DEFAULT
    write(*,*) 'Incorrect NVAR value in flux_roe: ', NVAR
    stop
END SELECT

!
! Step 2 : Compute Roe-averaged data from left and right states
!   These averages will be the input variables to the eigen problem
!


sqrtdl = sqrt(dl)
sqrtdr = sqrt(dr)
droe   = sqrtdl*sqrtdr
vxroe  = (sqrtdl*vxl + sqrtdr*vxr)/(sqrtdl+sqrtdr)
vyroe  = (sqrtdl*vyl + sqrtdr*vyr)/(sqrtdl+sqrtdr)
vzroe  = (sqrtdl*vzl + sqrtdr*vzr)/(sqrtdl+sqrtdr)

SELECT CASE (NVAR)
  CASE (7)
    byroe  = (sqrtdr*byl + sqrtdl*byr)/(sqrtdl+sqrtdr)
    bzroe  = (sqrtdr*bzl + sqrtdl*bzr)/(sqrtdl+sqrtdr)
    hroe   = ((el+pl+pbl)/sqrtdl+(er+pr+pbr)/sqrtdr)/(sqrtdl+sqrtdr)
    Xfactor= 0.5*( (byroe**2-byl*byr)+(bzroe**2-bzl*bzr))/droe
    Yfactor= 0.5*(dl+dr)/droe
  CASE (6)
    byroe  = (sqrtdr*byl + sqrtdl*byr)/(sqrtdl+sqrtdr)
    bzroe  = (sqrtdr*bzl + sqrtdl*bzr)/(sqrtdl+sqrtdr)
    Xfactor= 0.5*( (byroe**2-byl*byr)+(bzroe**2-bzl*bzr))/droe
    Yfactor= 0.5*(dl+dr)/droe
  CASE (5)
    hroe   = ((el+pl)/sqrtdl+(er+pr)/sqrtdr)/(sqrtdl+sqrtdr)
  CASE (4)
END SELECT 


!
! Step 3 : Compute eigenvalues and eigenmatrices from Roe-averaged values
!

SELECT CASE (NVAR)
  CASE (7)
     call eigen_adb_mhd_cons(NX,NVAR,gamma,droe,vxroe,vyroe,vzroe,hroe,&
          Bx,byroe,bzroe,Xfactor,Yfactor,lambda,rem,lem)
  CASE (6)
     call eigen_iso_mhd_cons(NX,NVAR,ISOTHERMAL_C,droe,vxroe,vyroe,vzroe,&
          Bx,byroe,bzroe,Xfactor,Yfactor,lambda,rem,lem)
  CASE (5)
     call eigen_adb_hyd_cons(NX,NVAR,gamma,vxroe,vyroe,vzroe,hroe,&
               lambda,rem,lem)
  CASE (4)
     call eigen_iso_hyd_cons(NX,NVAR,droe,vxroe,vyroe,vzroe, &
               lambda,rem,lem)
  CASE DEFAULT
END SELECT


!
! Step 4 : Create intermediate states from eigenmatrices
!          array a
! 
! Then check that the density  and pressure in the intermediate
! states are positive. If not, need to use the HLLE flux.
! Generally this shouldn't happen.
! Nothing beside the HLLE flag (logical) is used again
! from that block of code.
!


do i=1,NX
  a(i,:)= (Ur(i,1)-Ul(i,1))*lem(1,:,i)
  do n=2,NVAR
    a(i,:)=a(i,:)+(Ur(i,n)-Ul(i,n))*lem(n,:,i)
  enddo

  Uim(:) = Ul(i,:)
  Pim = 0.
  do n=1,NVAR
    Uim(:) = Uim(:) + a(i,n)*rem(n,:,i)
  enddo
  if (adiabatic) then 
    Pim = Uim(5) - .5*(Uim(2)**2+Uim(3)**2+Uim(4)**2)/Uim(1)
    if (mhd) Pim = Pim - .5*(Uim(NVAR)**2+Uim(NVAR-1)**2+Bx(i)**2)
  endif
  if (Uim(1) .lt. 0.) then
     HLLE(i)=.true.
     write(*,*)' HLLE at zone ', i,' Uim(1) = ',Uim(1)
  else if (Pim .lt. 0.) then
     HLLE(i) = .true.
     write(*,*)' HLLE at zone ', i,' Pim = ',Pim
  else
     HLLE(i) = .false.
  endif
enddo

!
!
! Step 5 : Compute L/R fluxes
!  These are computed from the left and right input primitive variables
!

fxl(:,1) = mxl
fxr(:,1) = mxr

fxl(:,2) = mxl*vxl
fxr(:,2) = mxr*vxr

fxl(:,3) = myl*vxl
fxr(:,3) = myr*vxr

fxl(:,4) = mzl*vxl
fxr(:,4) = mzr*vxr

if (adiabatic) then
  fxl(:,2) = fxl(:,2) + pl
  fxr(:,2) = fxr(:,2) + pr
  fxl(:,5) = (el+pl)*vxl
  fxr(:,5) = (er+pr)*vxr
else
  fxl(:,2) = fxl(:,2) + dl*ISOTHERMAL_C**2
  fxr(:,2) = fxr(:,2) + dr*ISOTHERMAL_C**2
endif

if (mhd) then
  fxl(:,2) = fxl(:,2) - .5*(Bx**2-byl**2-bzl**2)
  fxr(:,2) = fxr(:,2) - .5*(Bx**2-byr**2-bzr**2)

  fxl(:,3) = fxl(:,3) - Bx*byl
  fxr(:,3) = fxr(:,3) - Bx*byr

  fxl(:,4) = fxl(:,4) - Bx*bzl
  fxr(:,4) = fxr(:,4) - Bx*bzr

  if (adiabatic) then
     fxl(:,5) = fxl(:,5) + (pbl*vxl-Bx*(Bx*vxl+byl*vyl+bzl*vzl))
     fxr(:,5) = fxr(:,5) + (pbr*vxr-Bx*(Bx*vxr+byr*vyr+bzr*vzr))
  endif

  fxl(:,NVAR-1) = byl*vxl - Bx*vyl
  fxr(:,NVAR-1) = byr*vxr - Bx*vyr
  fxl(:,NVAR)   = bzl*vxl - Bx*vzl
  fxr(:,NVAR)   = bzr*vxr - Bx*vzr
endif

!
! Step 6 : Compute fluxes at interface using either Roe of HLLE solver
!


computeflux: do i=1,NX

  if (HLLE(i)) then       !  HLLE flux computation
    if (adiabatic) then
      csl = gamma*pl(i)/dl(i)
      csr = gamma*pr(i)/dl(i)
    else
      csl = ISOTHERMAL_C**2
      csr = ISOTHERMAL_C**2
    endif
    if (mhd) then
      vaxsq = Bx(i)**2/dl(i)
      ct2   = (byl(i)**2+bzl(i)**2)/dl(i)
    else
      vaxsq = 0. 
      ct2 = 0.
    endif
    cfsq = .5*(vaxsq+ct2+csl+sqrt((vaxsq+ct2-csl)**2+4.*csl*ct2))
    cfl  = sqrt(cfsq)  

    if (mhd) then
      vaxsq = Bx(i)**2/dr(i)
      ct2   = (byr(i)**2+bzr(i)**2)/dr(i)
    else
      vaxsq = 0. 
      ct2 = 0.
    endif
    cfsq = .5*(vaxsq+ct2+csr+sqrt((vaxsq+ct2-csr)**2+4.*csr*ct2))
    cfr  = sqrt(cfsq)  

    bp  = MAX( MAX(lambda(NVAR,i),(vxr(i)+csr)), 0._r_kind)
    bm  = MIN( MIN(lambda(1,i),(vxl(i)-csl)),0._r_kind)
    fluxd(i)  = ((bp*fxl(i,1)-bm*fxr(i,1))+(bp*bm)*(cdr(i)-cdl(i)))/(bp-bm)
    fluxm1(i) = ((bp*fxl(i,2)-bm*fxr(i,2))+(bp*bm)*(mxr(i)-mxl(i)))/(bp-bm)
    fluxm2(i) = ((bp*fxl(i,3)-bm*fxr(i,3))+(bp*bm)*(myr(i)-myl(i)))/(bp-bm)
    fluxm3(i) = ((bp*fxl(i,4)-bm*fxr(i,4))+(bp*bm)*(mzr(i)-mzl(i)))/(bp-bm)

    if (adiabatic) &
    fluxe(i)  = ((bp*fxl(i,5)-bm*fxr(i,5))+(bp*bm)*(er(i)-el(i)))/(bp-bm)

    if (mhd) then
       fluxby(i) = ((bp*fxl(i,NVAR-1)-bm*fxr(i,NVAR-1)) &
                   +(bp*bm)*(byr(i)-byl(i)))/(bp-bm)
       fluxbz(i) = ((bp*fxl(i,NVAR)-bm*fxr(i,NVAR)) &
                   +(bp*bm)*(bzr(i)-bzl(i)))/(bp-bm)
    endif

  else                    ! Roe flux computation

    fluxd(i)  = .5*(fxl(i,1)+fxr(i,1))
    fluxm1(i) = .5*(fxl(i,2)+fxr(i,2))
    fluxm2(i) = .5*(fxl(i,3)+fxr(i,3))
    fluxm3(i) = .5*(fxl(i,4)+fxr(i,4))
    if (adiabatic) fluxe(i) = .5*(fxl(i,5)+fxr(i,5))
    if (mhd) then
       fluxby(i) = .5*(fxl(i,NVAR-1)+fxr(i,NVAR-1))
       fluxbz(i) = .5*(fxl(i,NVAR)+fxr(i,NVAR))
    endif
    
    do n=1,NVAR
      fluxd(i)  = fluxd(i)  - .5*ABS(lambda(n,i))*a(i,n)*rem(n,1,i)
      fluxm1(i) = fluxm1(i) - .5*ABS(lambda(n,i))*a(i,n)*rem(n,2,i)
      fluxm2(i) = fluxm2(i) - .5*ABS(lambda(n,i))*a(i,n)*rem(n,3,i)
      fluxm3(i) = fluxm3(i) - .5*ABS(lambda(n,i))*a(i,n)*rem(n,4,i)
      if (adiabatic) then
         fluxe(i)  = fluxe(i) - .5*ABS(lambda(n,i))*a(i,n)*rem(n,5,i)
      endif
      if (mhd) then
         fluxby(i)  = fluxby(i) - .5*ABS(lambda(n,i))*a(i,n)*rem(n,NVAR-1,i)
         fluxbz(i)  = fluxbz(i) - .5*ABS(lambda(n,i))*a(i,n)*rem(n,NVAR,i)
      endif
    enddo
  endif

end do computeflux

!
! find maximum wavespeed 
!

maxev = MAX( MAXVAL(ABS(lambda(1,:))),MAXVAL(ABS(lambda(NVAR,:))))

!
! clean up and return
!


if (adiabatic) deallocate(hroe)
if (mhd) deallocate(pbl,pbr,byroe,bzroe,Xfactor, Yfactor)

NULLIFY(fluxd,fluxm1,fluxm2,fluxm3)
NULLIFY(dr,dl,vxr,vxl,vyr,vyl,vzr,vzl)
NULLIFY(cdr,cdl,mxr,mxl,myr,myl,mzr,mzl)

if (adiabatic) NULLIFY(fluxe,pr,pl,er,el)

end subroutine flux_roe
