;
; NAME: 
;    ATHPLOT
;
; PURPOSE:
;    The ATHPLOT command produces plots of the conserved U or primative
;    W variables produced by the athena-1D code.  The command is intended
;    to provide quick and flexible multi-plot output for 1D simulations.
;    The command also accepts plotting keywords, which are applied to
;    all the plots produced.  For example, ATHPLOT,x,u,PSYM=4 will
;    produce plots with diamonds rather than connected lines.
;    
; 
; AUTHOR:
;     John Hawley, University of Virginia
;     This work supported by NSF Grant AST-0113571 
;
; CALLING SEQUENCE:
;
;     ATHPLOT, x, u, [/CONS], [/PRIM], [INCLUDE= Intarray]
;
; INPUTS:
;
;     x:  A real array, x(nx) of grid cell locations x for nx zones.
;
;     u:  A real array u(nx,nvar)  of conserved or primative variables
;         produced by athena-1d.  The nvar dimension depends on the physics 
;         of the problem.  For isothermal hydrodynamics nvar=4.  For adiabatic 
;         hydro, nvar=5.  For MHD nvar=7 (isothermal) or nvar=8 (adiabatic).
;         The conserved variable array is 
;         U = (rho, rho u_x, rho u_y, rho u_z, E, B_y, B_z, B_x).
;         The primative variable array is
;         W = (rho, u_x, u_y, u_z, P, B_y, B_z, B_x).
;
;
; KEYWORDS:
;     /CONS: When set, /CONS instructs the plotting routine to use labels
;            appropriate for the conserved variables.  (DEFAULT)
;
;     /PRIM: When set, /PRIM instructs the plotting routine to use labels
;            appropriate for the primative variables.  If both /CONS and /PRIM
;            are set, the plots are labeled using conservative variable names.
;
;     INCLUDE=intarr: 
;            Set this keyword to an integer vector of up to eight 
;            elements specifying which variables should be plotted.  A zero in 
;            the nth location means do not plot the nth variable of U.  For 
;            example, in the Sod shock tube problem there are no transverse 
;            momenta, so setting INCLUDE=[1,1,0,0,1] would plot only 
;            rho, (rho u_x) and E from the array U.  
;            If the length of INCLUDE is less than $nvar$, the
;            remaining values are set to zero and the plots suppressed.
;            If INCLUDE is longer than $nvar$ the extra values are ignored.
;
;
; LICENSE:
;
; Copyright 2003 James M. Stone, Thomas Gardiner,
;                Peter Teuben, John F. Hawley
;
;   This program is free software; you can redistribute it and/or modify
;   it under the terms of the GNU General Public License as published by
;   the Free Software Foundation.
;
;   This program is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;   GNU General Public License for more details.
;
;
; HISTORY
; 
;    Written by John F. Hawley, August 2003
;
;#######################################################################
;

PRO athplot,x,a,CONS=consvar,PRIM=primvar,INCLUDE=IncludeList,_Extra=extra

  On_Error, 1

  iplot = INTARR(8)
  iplot[0:7] = 1
  numParams= N_Params()
  IF numParams LT 2 THEN $
     message, 'Two parameters needed in athplt:  x, u'

  IF Keyword_Set(primvar) THEN $
     IF Keyword_Set(consvar) THEN print,'Both PRIM and CONS set:  Assuming CONS'

  nip = N_Elements(IncludeList)
  IF (nip NE 0) THEN iplot[*]=0
  IF (nip GT 8) THEN nip=8
  IF (nip GT 0) THEN iplot[0:nip-1] = IncludeList[0:nip-1]

  np2 = N_Elements(WHERE(iplot NE 0))
  nx = N_Elements(x)
  na = SIZE(a)
  if (nx NE na[1])  THEN $
     message, 'Improper array size of U or W array'
  if (N_Elements(na) EQ 5) THEN nvar = na[2] ELSE nvar=1

  yname = StrArr(8)
  yname[0]='Density'
  yname[5]='Y Field'
  yname[6]='Z Field'
  yname[7]='X Field'

  IF Keyword_Set(primvar) THEN BEGIN
    yname[1]='Vx'
    yname[2]='Vy'
    yname[3]='Vz'
    yname[4]='Pressure'
  ENDIF ELSE BEGIN
    yname[1]='X Momentum'
    yname[2]='Y Momentum'
    yname[3]='Z Momentum'
    yname[4]='Energy'
  ENDELSE
  IF Keyword_Set(consvar) THEN BEGIN
    yname[1]='X Momentum'
    yname[2]='Y Momentum'
    yname[3]='Z Momentum'
    yname[4]='Energy'
  ENDIF

  CASE nvar OF
     4:  nplots=4
     5:  nplots=5
     7:  BEGIN
           nplots=7
           yname[4]=yname[5]
           yname[5]=yname[6]
         END
     8: nplots=8
     1:  BEGIN
           nplots=1
           yname[0] = ' '
         END
     ELSE:    $
          message, 'Incorrect NVAR value in U or W array'
  ENDCASE
  IF (np2 LT nplots) THEN nplots=np2

  IF (nplots GT 4) THEN BEGIN 
     !P.MULTI=[0,2,3]
     IF (nplots GT 6) THEN !P.MULTI=[0,3,3]
  ENDIF ELSE BEGIN
     !P.MULTI=[0,2,2]
     IF (nplots EQ 1) THEN !P.MULTI=0
  ENDELSE


  FOR i=0,nvar-1 DO BEGIN
      IF (iplot[i] NE 0) THEN plot,x,a[*,i],xtit='X',ytit=yname(i),_Extra=extra
  ENDFOR

  !P.MULTI=0
END
