;
; NAME: 
;     HISTPLOT
;
; PURPOSE:
;     The HISTPLOT command produces representative plots from the
;     athena-1D history data.  The command is intended
;     to provide quick multi-plot output for 1D simulations.
; 
; AUTHOR:
;     John Hawley, University of Virginia
;     This work supported by NSF Grant AST-0113571 
; 
; CALLING SEQUENCE:
;
;     HISTPLOT, hist
;
; INPUTS:
;     A float array hist(nvar,n) of n history data dumps
;     produced by athena-1d and read in by HISTPLOT.
;     nvar = 6 for isothermal hydro
;     nvar = 7 for adiabatic hydro
;     nvar = 9 for isothermal mhd
;     nvar = 10 for adiabatic mhd
;
; KEYWORDS:
;     The command also accepts plotting keywords, which are applied to
;     all the plots produced.  For example, HISTPLOT,hist,PSYM=4 will
;     produce plots with diamonds rather than connected lines.
;
; LICENSE:
;
; Copyright 2003 James M. Stone, Thomas Gardiner,
;                Peter Teuben, John F. Hawley
;
;   This program is free software; you can redistribute it and/or modify
;   it under the terms of the GNU General Public License as published by
;   the Free Software Foundation.
;
;   This program is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;   GNU General Public License for more details.
;
;
; HISTORY
; 
;    Written by John F. Hawley, August 2003
;
;#######################################################################

PRO histplot,hist,_Extra=extra

  On_Error, 1

  numParams= N_Params()
  IF numParams LT 1 THEN $
     message, 'Must supply history array to histplot  '


  na = SIZE(hist)
  CASE na[0] OF
     0: message, 'History variable is a scalar or undefined'
     1: BEGIN
        print,'History variable is rank 1 vector.  Plotting...'
        plot,hist,_Extra=extra
        return
        END
     ELSE: BEGIN
           nx = na[2]
           nvar = na[1]
           END
  ENDCASE

  time = REFORM(hist[0,*])
  dt   = REFORM(hist[1,*])
  mass = REFORM(hist[2,*])

  CASE nvar OF
     6:  BEGIN
           xke = REFORM(hist[3,*])
           yke = REFORM(hist[4,*])
           zke = REFORM(hist[5,*])
           !P.MULTI=[0,2,2]
           plot,time,dt,xtit='time',ytit='timestep',_Extra=extra
           plot,time,mass-mass[0],xtit='time',ytit='Change in Mass',_Extra=extra
           kemax=max(hist[3:5,*]) 
           plot,time,xke,xtit='time',yr=[0,kemax],ytit='Kinetic Energy',_Extra=extra
              IF (max(yke) GT 0.) THEN oplot,time,yke,linestyle=2
              IF (max(zke) GT 0.) THEN oplot,time,zke,linestyle=3
           plot,time,mass,xtit='time',ytit='Thermal Energy',_Extra=extra
         END

     7:  BEGIN
           ene = REFORM(hist[3,*])
           xke = REFORM(hist[4,*])
           yke = REFORM(hist[5,*])
           zke = REFORM(hist[6,*])
           therm = ene-xke-yke-zke
           !P.MULTI=[0,2,2]
           plot,time,dt,xtit='time',ytit='timestep',_Extra=extra
           plot,time,mass-mass[0],xtit='time',ytit='Change in Mass',_Extra=extra
           kemax=max(hist[4:6,*]) 
           plot,time,xke,xtit='time',yr=[0,kemax],ytit='Kinetic Energy',_Extra=extra
              IF (max(yke) GT 0.) THEN oplot,time,yke,linestyle=2
              IF (max(zke) GT 0.) THEN oplot,time,zke,linestyle=3
           plot,time,therm,xtit='time',ytit='Thermal Energy',_Extra=extra
         END

     9:  BEGIN
           xke = REFORM(hist[3,*])
           yke = REFORM(hist[4,*])
           zke = REFORM(hist[5,*])
           xmg = REFORM(hist[6,*])
           ymg = REFORM(hist[7,*])
           zmg = REFORM(hist[8,*])
           !P.MULTI=[0,2,3]
           plot,time,dt,xtit='time',ytit='timestep',_Extra=extra
           plot,time,mass-mass[0],xtit='time',ytit='Change in Mass',_Extra=extra
           kemax=max(hist[4:6,*]) 
           plot,time,xke,xtit='time',yr=[0,kemax],ytit='Kinetic Energy',_Extra=extra
              IF (max(yke) GT 0.) THEN oplot,time,yke,linestyle=2
              IF (max(zke) GT 0.) THEN oplot,time,zke,linestyle=3
           plot,time,mass,xtit='time',ytit='Thermal Energy',_Extra=extra
           plot,time,xmg,xtit='time',ytit='X Magnetic Energy',_Extra=extra
           mgmax = max(hist[7:8,*])
           plot,time,ymg,xtit='time',yr=[0,mgmax],ytit='Y, Z Magnetic Energy',_Extra=extra
              IF (max(zmg) GT 0.) THEN oplot,time,zmg,linestyle=3
         END

     10: BEGIN
           ene = REFORM(hist[3,*])
           xke = REFORM(hist[4,*])
           yke = REFORM(hist[5,*])
           zke = REFORM(hist[6,*])
           xmg = REFORM(hist[7,*])
           ymg = REFORM(hist[8,*])
           zmg = REFORM(hist[9,*])
           therm = ene-xke-yke-zke-xmg-ymg-zmg
           !P.MULTI=[0,2,3]
           plot,time,dt,xtit='time',ytit='timestep',_Extra=extra
           plot,time,mass-mass[0],xtit='time',ytit='Change in Mass',_Extra=extra
           kemax=max(hist[4:6,*]) 
           plot,time,xke,xtit='time',yr=[0,kemax],ytit='Kinetic Energy',_Extra=extra
              IF (max(yke) GT 0.) THEN oplot,time,yke,linestyle=2
              IF (max(zke) GT 0.) THEN oplot,time,zke,linestyle=3
           plot,time,therm,xtit='time',ytit='Thermal Energy',_Extra=extra
           plot,time,xmg,xtit='time',ytit='X Magnetic Energy',_Extra=extra
           mgmax = max(hist[8:9,*])
           plot,time,ymg,xtit='time',yr=[0,mgmax],ytit='Y, Z Magnetic Energy',_Extra=extra
              IF (max(zmg) GT 0.) THEN oplot,time,zmg,linestyle=3
         END

     ELSE:    $
          message, 'Incorrect NVAR value history array'
  ENDCASE


  !P.MULTI=0
END
