;
; NAME: 
;
;   READBIN
;
; PURPOSE:
;
;   The READBIN command reads the binary output from the Athena 1D
;   program.  Data is read from the given filename and placed into the 
;   grid array and the U and W arrays, corresponding to the conserved and
;   primative variables of the Athena-1D code, respectively.
; 
; AUTHOR:
;     John Hawley, University of Virginia
;     This work supported by NSF Grant AST-0113571 
; 
; CALLING SEQUENCE:
;
;   READBIN, filename, x, u, w, [GAMMA = variable], ISOC = variable],
;            [/FORTRAN], [ENDIAN$=string]
;
; INPUTS:
;   filename: a scalar string containing the name of the binary file to be
;   read by READBIN.
;
; OUTPUTS:
;
;   x:  A real array, x(nx$ of grid cell locations x for nx zones
;   returned by READBIN.
;
;   u: A real array u(nx,nvar)  of conserved variables returned by READBIN.
;   The nvar dimension depends on the physics of the problem.  For
;   isothermal hydrodynamics nvar=4.  For adiabatic hydro, nvar=5.
;   For MHD nvar=7 (isothermal) or nvar=8 (adiabatic).
;   U = rho, rho u_x, rho u_y, rho u_z, E, B_y, B_z, B_x.
;
;   w: A real array w(nx,nvar)  of primative variables returned by READBIN.
;   The nvar dimension depends on the physics of the problem.  For
;   isothermal hydrodynamics nvar=4.  For adiabatic hydro, nvar=5.
;   For MHD nvar=7 (isothermal) or nvar=8 (adiabatic).
;   W = rho, u_x, u_y,  u_z, P, B_y, B_z, B_x.
;
;
; KEYWORDS:
;
;   GAMMA: When set READBIN will return the value of the adiabatic gamma to
;   the named variable.
;
;   ISOC: When set READBIN will return the value of the isothermal sound speed
;   to the named variable.
;
;   FORTRAN: Set this keyword if the binary file to be read was produced by a
;   fortran program.  With this keyword set, a four byte offset for the 
;   header and footer of an fortran binary write is included in the read.
;
;   ENDIAN: Set this keyword to one of three string values: ``big'',
;   ``little'' or ``native'' which specifies the byte ordering of the 
;   file to be read.  If the computer running IDL uses byte ordering 
;   that is different from that of the file, this will swap the order 
;   of the bytes in the read.
;   (Default:  ``native'' $=$ perform no byte swapping.)
;
;
; LICENSE:
;
; Copyright 2003 James M. Stone, Thomas Gardiner,
;                Peter Teuben, John F. Hawley
;
;   This program is free software; you can redistribute it and/or modify
;   it under the terms of the GNU General Public License as published by
;   the Free Software Foundation.
;
;   This program is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;   GNU General Public License for more details.
;
;
; HISTORY
; 
;    Written by John F. Hawley, August 2003
;
;#######################################################################

PRO readbin,fname,x,U,W,Fortran=offsetFortran,Gamma=gamma,IsoC=isoc,_Extra=extra

  On_Error, 1
  On_IOError, ReadProblem

  numParams= N_Params()
  IF numParams LT 4 THEN $
     message, 'Four parameters needed in readbin: filename, x, u, w'

  IF Keyword_Set(offsetFortran) THEN $
     offset = 4  ELSE $
     offset = 0

  ndata=lonarr(2)
  eos = fltarr(2)
  openr,lun,fname, /Get_Lun

  ndata=READ_BINARY(lun,DATA_START=offset,DATA_TYPE=3,DATA_DIMS=2,_Extra=extra)
  eos=READ_BINARY(lun,DATA_START=3*offset+8,DATA_TYPE=4,DATA_DIMS=2,_Extra=extra)
  nx=ndata[0]
  nvar=ndata[1]
  gamma = eos(0)+1.
  isoc  = eos(1)

  x=fltarr(nx)
  u=fltarr(nx,nvar)
  w=fltarr(nx,nvar)
  
  d=fltarr(nx)
  m1=fltarr(nx)
  m2=fltarr(nx)
  m3=fltarr(nx)
  x=read_binary(lun,data_start=(5*offset+16),data_type=4,data_dim=nx,_Extra=extra)
  d=read_binary(lun,data_start=(7*offset+16+nx*4),data_type=4,data_dim=nx,_Extra=extra)
  m1=read_binary(lun,data_start=(9*offset+16+2*nx*4),data_type=4,data_dim=nx,_Extra=extra)
  m2=read_binary(lun,data_start=(11*offset+16+3*nx*4),data_type=4,data_dim=nx,_Extra=extra)
  m3=read_binary(lun,data_start=(13*offset+16+4*nx*4),data_type=4,data_dim=nx,_Extra=extra)

  u(*,0) = d
  u(*,1) = m1
  u(*,2) = m2
  u(*,3) = m3

  w(*,0) = d
  w(*,1) = m1/d
  w(*,2) = m2/d
  w(*,3) = m3/d

  CASE nvar OF
     4:
     5:  BEGIN
           e=fltarr(nx)
           e=read_binary(lun,data_start=(15*offset+16+5*nx*4),data_type=4,data_dim=nx,_Extra=extra)
           u(*,4) = e
           w(*,4) = (e - .5*(m1^2+m2^2+m3^2)/d)*eos[0]
         END
     7:  BEGIN
           b1=fltarr(nx)
           b2=fltarr(nx)
           b3=fltarr(nx)
           b2=read_binary(lun,data_start=(15*offset+16+5*nx*4),data_type=4,data_dim=nx,_Extra=extra)
           b3=read_binary(lun,data_start=(17*offset+16+6*nx*4),data_type=4,data_dim=nx,_Extra=extra)
           b1=read_binary(lun,data_start=(19*offset+16+7*nx*4),data_type=4,data_dim=nx,_Extra=extra)
           u(*,4) = b2
           u(*,5) = b3
           u(*,6) = b1
           w(*,4) = b2
           w(*,5) = b3
           w(*,6) = b1
         END
     8:  BEGIN
           e=fltarr(nx)
           e=read_binary(lun,data_start=(15*offset+16+5*nx*4),data_type=4,data_dim=nx,_Extra=extra)
           b1=fltarr(nx)
           b2=fltarr(nx)
           b3=fltarr(nx)
           b2=read_binary(lun,data_start=(17*offset+16+6*nx*4),data_type=4,data_dim=nx,_Extra=extra)
           b3=read_binary(lun,data_start=(19*offset+16+7*nx*4),data_type=4,data_dim=nx,_Extra=extra)
           b1=read_binary(lun,data_start=(21*offset+16+8*nx*4),data_type=4,data_dim=nx,_Extra=extra)
           u(*,4) = e
           w(*,4) = (e - .5*(m1^2+m2^2+m3^2)/d - .5*(b2^2+b3^2))*eos[0]
           u(*,5) = b2
           u(*,6) = b3
           u(*,7) = b1
           w(*,5) = b2
           w(*,6) = b3
           w(*,7) = b1
         END
  ENDCASE

  free_lun, lun
  RETURN

  ReadProblem: Print, 'Problem reading data.  Check file name: ', fname
  if N_Elements(lun) NE 0 THEN Free_Lun, lun
  RETURN

END
