;
;
; NAME: 
;    READHIST
;
; PURPOSE:
;
;   READHIST reads history data from an input ASCII file, placing the 
;   data in an output variable named hist
;   The number of variables per line depends on physics
;   10:  mhd+adiabatic
;    9:  mhd isothermal
;    7:  hydro adiabatic
;    6:  hydro isothermal
;
; 
; AUTHOR:
;     John Hawley, University of Virginia
;     This work supported by NSF Grant AST-0113571 
;
; CALLING SEQUENCE:
;
;    READHIST, filename, hist
;
;
; INPUTS:
;
;    filename:  A scalar string containing the name of the history file 
;               to be read by READHIST, e.g., ``Sod.hst''.
;
;    hist:  A real array, hist(nvar,n) of nvar history variables over n timesteps
;           returned by READHIST.  The number nvar depends on the physics.  
;           The READHIST routine counts the number of variables on
;           a line to determine this value.
;
;
; KEYWORDS:
;
; LICENSE:
;
; Copyright 2003 James M. Stone, Thomas Gardiner,
;                Peter Teuben, John F. Hawley
;
;   This program is free software; you can redistribute it and/or modify
;   it under the terms of the GNU General Public License as published by
;   the Free Software Foundation.
;
;   This program is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;   GNU General Public License for more details.
;
;
; HISTORY
; 
;    Written by John F. Hawley, August 2003
;
;#######################################################################

PRO readhist,file,hist

  On_Error, 1

  numParams= N_Params()
  IF numParams LT 2 THEN $
     message, 'Two parameters needed in read_hist: filename, hist'

  openr,lun,file,/get_lun, ERROR = err
  IF (err NE 0) THEN BEGIN
    print,'Error opening history file: ', file
    return
  ENDIF
  free_lun,lun

; How this routine determines number of history lines and NVAR depends on whether
; or not the Unix command wc is available.

 IF !VERSION.OS_FAMILY EQ 'unix' THEN BEGIN
         if strpos(file,'~') GE 0 then file = expand_tilde(file)
         spawn,'wc -l  < '+file, result, /sh
         nlines = long(result[0])
         spawn,'wc -w  < '+file, result, /sh
         nwords= long(result[0])
         nvars=nwords/nlines
         IF (nvars*nlines NE nwords) THEN BEGIN
           print,'Error: nvar*nline NE nword in  history file: ', file
           return
         ENDIF

 ENDIF ELSE BEGIN                 
; Option for non-Unix systems

        openr,lun,file,/get_lun, ERROR = err
        nlines = 0l
        tmp = ' '
         while not eof(lun) do begin
          readf,lun,tmp
          nlines = nlines + 1
          endwhile
         free_lun,lun
         nvars=10
         Catch, theError
         IF (theError NE 0) THEN BEGIN
            nvars=nvars-1
            IF (nvars LT 5) THEN BEGIN
               Catch, /cancel
               print, 'Problem with history: number of elements per line < 5 '
               return
            ENDIF
         ENDIF
         ftmp=fltarr(nvars)
         reads,tmp,ftmp
         Catch,/cancel
 ENDELSE

 hist=fltarr(nvars,nlines)

 OpenR, lun, file, /get_lun
 readf, lun, hist

 free_lun,lun

 print, nlines,' lines read ', nvars,' history variables per line'

END
