subroutine integrate_1step(NVAR,agrid,CourNo)
!
! Integrate_1step :  advances 1-D code conserved vector U one timestep by computing
!                    fluxes F(U) in the equation dU/dt + dF(U)/dx = 0
!                    Converved vector U is addressed through variable agrid
!                    Number of components of U is NVAR; this depends on physics
!
!        Isothermal hydro  NVAR = 4, U= rho, rho v_x, rho v_y, rho v_x
!        Adiabatic  hydro  NVAR = 5, U= rho, rho v_x, rho v_y, rho v_x, E
!        Isothermal mhd    NVAR = 6, U= rho, rho v_x, rho v_y, rho v_x, B_y, B_z
!        Adiabatic  mhd    NVAR = 7, U= rho, rho v_x, rho v_y, rho v_x, E, B_y, B_z
!   
! 
! AUTHORS:
!     James M. Stone, Princeton Univeristy
!     Thomas Gardiner, University of Maryland
!     Peter J. Teuben, University of Maryland
!     John Hawley, University of Virginia
!
!     This software was produced with support from NSF Grant AST-0113571 
!
! Copyright 2003 James M. Stone, Thomas Gardiner,
!                Peter Teuben, John F. Hawley
!
!   This program is free software; you can redistribute it and/or modify
!   it under the terms of the GNU General Public License as published by
!   the Free Software Foundation.
!
!   This program is distributed in the hope that it will be useful,
!   but WITHOUT ANY WARRANTY; without even the implied warranty of
!   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!   GNU General Public License for more details.
!

use athenadefs
IMPLICIT NONE
type (grid) agrid
integer, intent(IN) :: NVAR
real (r_kind), intent(IN) :: CourNo    !  Courant number

!
! LOCAL variables
!
! W : array of primative variables, viz.  density, velocity, magnetic fields
! Wleft  :  Left primative at zone interface, input to flux solver
! Wright :  Right primative at zone interface, input to flux solver
!
!

real (r_kind), dimension(:,:), target, allocatable :: W, Wleft, Wright, flux
real (r_kind), dimension(:), pointer:: fluxd, fluxe, fluxm1, &
                           fluxm2, fluxm3, fluxb1, fluxb2, fluxb3
real (r_kind) :: dtodx
real (r_kind) :: maxev_lr, maxev_flux, maxev_xswp
integer :: i

save W, Wleft, Wright, flux

!------------------------------------------------------------------------
!
! initialize local scalars
! allocate work arrays as needed
! set pointers
!

dtodx = agrid%dt/agrid%dx
maxev_xswp = 0.

if (.NOT. allocated(W)) allocate(W(agrid%nx1,NVAR))
if (.NOT. allocated(Wleft)) allocate(Wleft(agrid%nx1,NVAR))
if (.NOT. allocated(Wright)) allocate(Wright(agrid%nx1,NVAR))
if (.NOT. allocated(flux)) allocate(flux(agrid%nx1,NVAR))

fluxd=>flux(:,1); fluxm1=>flux(:,2); fluxm2=>flux(:,3); fluxm3=>flux(:,4)

SELECT CASE (NVAR)
  CASE (7)
    fluxe=>flux(:,5); fluxb2=>flux(:,6); fluxb3=>flux(:,7)
    W(:,1) = agrid%d
    W(:,2) = agrid%m1/W(:,1)
    W(:,3) = agrid%m2/W(:,1)
    W(:,4) = agrid%m3/W(:,1)
    W(:,5) = max((agrid%gamma-1.)*(agrid%e  &
                - .5*(agrid%b1**2+agrid%b2**2+agrid%b3**2) &
                - .5*agrid%d*(W(:,2)**2+W(:,3)**2+W(:,4)**2)), TINY_NUMBER)
    W(:,6) = agrid%b2
    W(:,7) = agrid%b3
  CASE (6)
    fluxb2=>flux(:,5); fluxb3=>flux(:,6)
    W(:,1) = agrid%d
    W(:,2) = agrid%m1/W(:,1)
    W(:,3) = agrid%m2/W(:,1)
    W(:,4) = agrid%m3/W(:,1)
    W(:,5) = agrid%b2
    W(:,6) = agrid%b3
  CASE (5)
    fluxe=>flux(:,5)
    W(:,1) = agrid%d
    W(:,2) = agrid%m1/W(:,1)
    W(:,3) = agrid%m2/W(:,1)
    W(:,4) = agrid%m3/W(:,1)
    W(:,5) = max((agrid%gamma-1._r_kind)&
            *(agrid%e-0.5_r_kind*agrid%d*(W(:,2)**2+W(:,3)**2+W(:,4)**2)), TINY_NUMBER)
  CASE (4)
    W(:,1) = agrid%d
    W(:,2) = agrid%m1/W(:,1)
    W(:,3) = agrid%m2/W(:,1)
    W(:,4) = agrid%m3/W(:,1)
  CASE DEFAULT
    write(*,*) 'Incorrect NVAR value in integrate_1step: ', NVAR
    stop

END SELECT

!
! compute left and right states, then pass them on
! to get fluxes using roe scheme
! Compute new timestep from fastest wavespeed seen in
! computing the states or the fluxes
!

call lr_states(agrid%nx1,NVAR,W,agrid%b1,agrid%gamma,dtodx,Wleft,Wright,maxev_lr)

call flux_roe(NVAR,agrid%nx1,agrid%gamma,agrid%b1,Wleft,Wright,flux,maxev_flux)

maxev_xswp = MAX(maxev_xswp,(MAX(maxev_lr,maxev_flux)))

!
! Update conserved variables using fluxes obtained above
!

evolve: do i=agrid%is, agrid%ie
   agrid%d(i) = agrid%d(i) - dtodx*(fluxd(i+1) - fluxd(i))
   agrid%m1(i) = agrid%m1(i) - dtodx*(fluxm1(i+1) - fluxm1(i))
   agrid%m2(i) = agrid%m2(i) - dtodx*(fluxm2(i+1) - fluxm2(i))
   agrid%m3(i) = agrid%m3(i) - dtodx*(fluxm3(i+1) - fluxm3(i))
   if (adiabatic) then
     agrid%e(i) = agrid%e(i) - dtodx*(fluxe(i+1) - fluxe(i))
   endif
   if (mhd) then
     agrid%b2(i) = agrid%b2(i) - dtodx*(fluxb2(i+1) - fluxb2(i))
     agrid%b3(i) = agrid%b3(i) - dtodx*(fluxb3(i+1) - fluxb3(i))
   endif
end do evolve

agrid%time=agrid%time+agrid%dt
agrid%dt = CourNo*agrid%dx/maxev_xswp

!
! clean up work arrays and pointers
!

NULLIFY(fluxd,fluxm1,fluxm2,fluxm3)
if (adiabatic) NULLIFY(fluxe)
if (mhd) NULLIFY(fluxb2,fluxb3)

end subroutine integrate_1step
