subroutine lr_states(NX, NVAR, W, Bx, gamma, dtodx, Wleft, Wright,maxev)
! 
! AUTHORS:
!     James M. Stone, Princeton Univeristy
!     Thomas Gardiner, University of Maryland
!     Peter J. Teuben, University of Maryland
!     John Hawley, University of Virginia
!
!     This software was produced with support from NSF Grant AST-0113571 
!
! Copyright 2003 James M. Stone, Thomas Gardiner,
!                Peter Teuben, John F. Hawley
!
!   This program is free software; you can redistribute it and/or modify
!   it under the terms of the GNU General Public License as published by
!   the Free Software Foundation.
!
!   This program is distributed in the hope that it will be useful,
!   but WITHOUT ANY WARRANTY; without even the implied warranty of
!   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!   GNU General Public License for more details.
!

use athenadefs
IMPLICIT NONE

!
! Subroutine lr_states
!
! Computes left- and right-states of PRIMITIVE variables 
!
! Input Arguments:
!   NX    = number of zones in sweep direction
!   NVAR  = number of variables in eigenvalue problem (determined by physics)
!   W     = PRIMATIVE variables at cell centers along 1-D slice
!           W=(d,Vx,Vy,Vz,P,By,Bz)
!   Bx    = magnetic field in sweep direction
!   gamma = adiabatic exponent
!   dtodx = dt/dx
!
! Output Arguments:
!
!   Wl,Wr = L/R-states of PRIMITIVE variables at cell faces over [is:ie+1]
!
!   maxevlr = maximum absolute value of eigenvalues over all zones
!
! input declarations:

integer, intent(IN) :: NX, NVAR
real (r_kind), intent(IN) :: dtodx, gamma
real (r_kind), dimension(NX), intent(IN) :: Bx
real (r_kind), dimension(NX,NVAR), intent(IN) :: W

real (r_kind), dimension(NX,NVAR), intent(OUT) :: Wleft, Wright
real (r_kind), intent(OUT) :: maxev

!
! locals:  automatic arrays
!
!  lambda = characteristic eigenvalue
!  lem    = left eigen matrix
!  rem    = right eigen matrix
!  
!  dWc    = centered slope of W
!  dW(lrg) = other slopes of W
!  dWm    = monotonic slope of W along characteristic
!  da?    = dW? slopes projected along characteristic
!

real (r_kind), dimension(NX,NVAR) ::  Wlv, Wrv, dW, W6, Wim1h
real (r_kind), dimension(NX,NVAR) :: dWc, dWl, dWr, dWg, dWm
real (r_kind), dimension(NX,NVAR) :: dac, dal, dar, dag, da
real (r_kind), dimension(NVAR,NX) :: lambda
real (r_kind), dimension(NVAR,NVAR,NX) :: lem, rem

real (r_kind) :: sigmal, sigmar

integer :: i, n, m

!----------------------------------------------------------------------------
!
! LR_STATES computes the values of the primitive variables at
! the left and right side of the zone interface for use in the
! approximate riemann solver.
!
! The variables and their positions on the grid are given as follows:
! W = NVAR primitive variables, zone average for (i)
! dW = various slopes, dWm located at center (i)
! W6 = PPM variable zone (i)
! Wlv = Value at left side of zone (i) : i-1/2
! Wrv = value at right side of zone (i) : i+1/2
!
!      i-1/2       i        1+1/2
!        |         .          |   
!        |         .          |   
!        |         W          |   
!        |         .          |   
!        |         .          |   
!        |         .          |   
!
!         Wlv(i)          Wrv(i)
!    Wleft(i)     dW(i)
!    Wright(i)
!
!
!
!--- Step 1.  ---------------------------------------------------------------
!
! Obtain eigenvalues and eigenvectors based on primitive variables
!


SELECT CASE (NVAR)
  CASE (7)
     call eigen_adb_mhd_prim(NX,NVAR,gamma,W,Bx,lambda,rem,lem)
  CASE (6)
     call eigen_iso_mhd_prim(NX,NVAR,ISOTHERMAL_C,W,Bx,lambda,rem,lem)
  CASE (5)
     call eigen_adb_hyd_prim(NX,NVAR,gamma,W,lambda,rem,lem)
  CASE (4)
     call eigen_iso_hyd_prim(NX,NVAR,W,lambda,rem,lem)
  CASE DEFAULT
END SELECT

!
!--- Step 2.  ---------------------------------------------------------------
!
!  Compute L/R-states of primitive variables over entire 1D slice.  
!  This requires integrating the interpolation polynomial to get 
!  L/R-states at cell faces.
!  First step is to compute slopes
!  Second step is to project those slopes along the characteristics 
!  
! Compute slopes

call get_slopes(NVAR,NX,W,dWc,dWl,dWr,dWg)

! Project difference of primitive variables along characteristics


do i=1,NX
  dac(i,:) = dWc(i,1)*lem(1,:,i)
  dal(i,:) = dWl(i,1)*lem(1,:,i)
  dar(i,:) = dWr(i,1)*lem(1,:,i)
  dag(i,:) = dWg(i,1)*lem(1,:,i)
  do m=2,NVAR
    dac(i,:) = dac(i,:) + dWc(i,m)*lem(m,:,i)
    dal(i,:) = dal(i,:) + dWl(i,m)*lem(m,:,i)
    dar(i,:) = dar(i,:) + dWr(i,m)*lem(m,:,i)
    dag(i,:) = dag(i,:) + dWg(i,m)*lem(m,:,i)
  enddo
enddo

! Apply monotonicity constraint to projections along characteristics

do n=1,NVAR
  do i=1,NX
     if (dal(i,n)*dar(i,n) .gt. 0.) then
         da(i,n) = SIGN(1._r_kind,dac(i,n))*&
                     MIN(2._r_kind*MIN(ABS(dal(i,n)),abs(dar(i,n))), &
                     MIN(.5_r_kind*abs(dac(i,n)),abs(dag(i,n))) )
     else
         da(i,n) = 0.
     endif
  enddo
enddo

! Compute monotonic slopes of primitive variables from projections

do i=1,NX
  dWm(i,:) = da(i,1)*rem(1,:,i)
  do m=2,NVAR
    dWm(i,:) = dWm(i,:) + da(i,m)*rem(m,:,i)
  enddo
enddo

!
! Use slope and linear function to get left and right
! zone boundary values
!
! choose one or the other here. <------------------
!
! Third order
!

! Wim1h = 0.5*(W + EOSHIFT(W,DIM=1,SHIFT=-1)) &
!           - (dWm - EOSHIFT(dWm,DIM=1,SHIFT=-1))/6.0
! Wlv = Wim1h
! Wrv = EOSHIFT(Wim1h,DIM=1,SHIFT=+1)

! Second order
!
  Wlv = W - 0.5_r_kind * dWm
  Wrv = W + 0.5_r_kind * dWm
!
!--------------------------------------------------
!
! More monotonicity checks
! Ensure that a_L and a_R lie between the cell-center values
!

Wlv = MAX( MIN(W,EOSHIFT(W,SHIFT=-1,DIM=1)), Wlv)
Wlv = MIN( MAX(W,EOSHIFT(W,SHIFT=-1,DIM=1)), Wlv)
Wrv = MAX( MIN(W,EOSHIFT(W,SHIFT=+1,DIM=1)), Wrv)
Wrv = MIN( MAX(W,EOSHIFT(W,SHIFT=+1,DIM=1)), Wrv)

!
! Monotonize again with C-W eqn 1.10
!

dWc = Wrv-Wlv
dWg = 6.0_r_kind*(W-.5_r_kind*(Wlv+Wrv))
dWl = dWc*(dWc-dWg)
dWr = dWc*(dWc+dWg)

do n=1,NVAR
  do i=1,NX
    if ( (Wrv(i,n)-W(i,n))*(W(i,n)-Wlv(i,n)) .le. 0.) then
       Wlv(i,n) = W(i,n)
       Wrv(i,n) = W(i,n)
    else if (dWl(i,n) .lt. 0.) then
       Wlv(i,n) = 3.0_r_kind * W(i,n) - 2.0_r_kind*Wrv(i,n)
    else if (dWr(i,n) .lt. 0.) then
       Wrv(i,n) = 3.0_r_kind * W(i,n) - 2.0_r_kind*Wlv(i,n)
    endif
  enddo
enddo

!
! Compute coefficients of interpolation parabola, C-W eqn 1.5
! These are called dW and W6
!

dW = Wrv - Wlv
W6 = 6._r_kind *(W - 0.5_r_kind * (Wlv + Wrv))


!
! Compute left and right state by integrating interpolation parabolae 
! over the domain of dependence defined by max (min) eigenvalue 
! The maximum eigenvalue will be the NVAR one:  v+cfastest
! The minimum eigenvalue will be the first one: v-cfastest
! C-W eqn. 1.12.  Note left (right) state at interface i+1/2 (i-1/2)
! i on the right (left) side of cell i
!

do i=1,NX
  sigmal =  TWO_THIRDS * MAX(lambda(NVAR,i),0._r_kind)*dtodx
  Wleft (i,:)  = Wrv(i,:) - 0.75_r_kind*sigmal*(dW(i,:) - (1.-sigmal)*w6(i,:))
  sigmar = -TWO_THIRDS * MIN(lambda(1,i),0._r_kind)*dtodx
  Wright(i,:)  = Wlv(i,:) + 0.75_r_kind*sigmar*(dW(i,:) + (1.-sigmar)*w6(i,:))
enddo

! Shift the left value back to its appropriate location
!

Wleft = EOSHIFT(Wleft,DIM=1,SHIFT=-1)

!
! Subtract amount of each wave m that does not reach the interface
! during the timestep (C-W eqn 3.5ff)
! Re-use variables dal and dar at this point since it is already allocated
!


do i=1,NX-1
  do n=1,NVAR-1
   if (lambda(n,i) .gt. 0.) then
     dal(i,n) = (lem(1,n,i))*0.5_r_kind*dtodx &
               * ( (lambda(NVAR,i)-lambda(n,i))*(dW(i,1)-W6(i,1)) &
                   + dtodx*TWO_THIRDS*W6(i,1) &
               *   (lambda(NVAR,i)**2 - lambda(n,i)**2) )
     do m=2,NVAR
       dal(i,n) = dal(i,n) +(lem(m,n,i))*0.5_r_kind*dtodx &
               * ( (lambda(NVAR,i)-lambda(n,i))*(dW(i,m)-W6(i,m)) &
                   + dtodx*TWO_THIRDS*W6(i,m) &
               *   (lambda(NVAR,i)**2 - lambda(n,i)**2) )
     enddo
     do m=1,NVAR
       Wleft(i+1,m) = Wleft(i+1,m)+dal(i,n)*rem(n,m,i)
     enddo
   endif
  enddo
enddo

!
! boundary value to avoid divisions by zero
! i=1 boundary values will have no consequence
!

Wleft(1,:) = Wleft(2,:)

do i=1,NX
  do n=2,NVAR
   if (lambda(n,i) .lt. 0.) then
     dar(i,n) =  (lem(1,n,i))*0.5_r_kind*dtodx &
             * ( (lambda(1,i)-lambda(n,i))*(dW(i,1)-W6(i,1)) &
                 + dtodx*TWO_THIRDS*W6(i,1) &
             *   (lambda(1,i)**2 - lambda(n,i)**2) )
     do m=2,NVAR
       dar(i,n) = dar(i,n) +(lem(m,n,i))*0.5_r_kind*dtodx &
               * ( (lambda(1,i)-lambda(n,i))*(dW(i,m)-W6(i,m)) &
                   + dtodx*TWO_THIRDS*W6(i,m) &
               *   (lambda(1,i)**2 - lambda(n,i)**2) )
     enddo
     do m=1,NVAR
       Wright(i,m) = Wright(i,m)+dar(i,n)*rem(n,m,i)
     enddo
   endif
  enddo
enddo

!
! Maximum wavespeed for use later in limiting timestep
!

maxev = MAX( MAXVAL(ABS(lambda(1,:))),MAXVAL(ABS(lambda(NVAR,:))))

!
! End of subroutine
!


end subroutine lr_states
