PROGRAM athena
!
! ATHENA - Astrophysical MHD Roe solver
! Developed by J. M. Stone, T. Gardiner, P. Teuben, J. Hawley
!
! ATHENA:  f95 version of the 1-D roe scheme MHD code
! Fortran 95 version 1.0
! Fortran 95 verstion by J. Hawley
! 
! AUTHORS:
!     James M. Stone, Princeton Univeristy
!     Thomas Gardiner, University of Maryland
!     Peter J. Teuben, University of Maryland
!     John Hawley, University of Virginia
!
!     This software was produced with support from NSF Grant AST-0113571 
!
! Copyright 2003 James M. Stone, Thomas Gardiner,
!                Peter Teuben, John F. Hawley
!
!   This program is free software; you can redistribute it and/or modify
!   it under the terms of the GNU General Public License as published by
!   the Free Software Foundation.
!
!   This program is distributed in the hope that it will be useful,
!   but WITHOUT ANY WARRANTY; without even the implied warranty of
!   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!   GNU General Public License for more details.
!

use athenadefs

IMPLICIT NONE
!
type (grid) grid0

character(len=20) :: problem_id, restart_file
integer :: restart_flag
integer :: nlim
integer :: nx1, ibc_x1, obc_x1
real (r_kind) :: cour_no, dt_res, tlim, dt_hst, dt_bin, dt_hdf, dt_tab
real (r_kind) :: x1min, x1max

integer :: NVAR
integer :: i, n, nstep

integer, dimension(3) :: clock=0
integer:: tstart, tstop, now
real (r_kind)  :: elapsed
real (r_kind)  :: cputime

integer :: nframe, ihdfdump, ihdflast, itabdump, itablast &
                 , ibindump, ibinlast, ihstdump, ihstlast
!
! namelist variables
!
! job:  problem id - a string with the name of the run
!       restart_flag - allows for restart rather than new problem NOT IMPLEMENTED
!       restart_file - file to use as restart NOT IMPLEMENTED
!       mhd - logical set to .true. for MHD physics, .false. for hydro
!       adiabatic - logical set to .true. for adiabatic, .false for isothermal
!
! time: cour_no - courant number, should be < 1
!       dt_res - time interval for making a restart dump NOT IMPLEMENTED
!       nlim - run no more than this number of timesteps
!       tlim - end on this evolution time
!       dt_hst - time interval for history dump.  If =0 no history dump
!       dt_bin - time interval for binary data dump.  If =0 no binary dump
!       dt_hdf - time interval for HDF data dump. NOT IMPLEMENTED
!       dt_tab - time interval for ascii tabular data dump.  If =0 no tab dump
!
! grd:  nx1 - number of physical zones to use in 1D problem
!       x1min - minimum X value for grid
!       x1max - maximum X value for grid.
!       ibc_x1 - inner X boundary flag
!       obc_x1 - outer X boundary flag
!                1 = reflecting; 2 = outflow; 4 = periodic
!

namelist /job/ problem_id, restart_flag, restart_file, mhd, adiabatic
namelist /time/ cour_no, dt_res, nlim, tlim, dt_hst, dt_bin,dt_hdf, dt_tab
namelist /grd/ nx1, x1min, x1max, ibc_x1, obc_x1
!
!-----------------------------------------------------------------------------------
!
! Begin main routine
! read in namelist information on job
!
open(unit=12, file='athena.input',status='OLD',ERR=901)
read(12,nml=JOB,ERR=902)
read(12,nml=TIME,ERR=903)
read(12,nml=GRD,ERR=904)
close(12)

write(*,*) 'Problem id is: ',problem_id
write(*,*) 'number of cells: ', nx1
write(*,*) 'courant number: ',cour_no

if (mhd) then
  if (adiabatic) then
     write(*,*) 'Physics:  Adiabatic MHD'
  else
     write(*,*) 'Physics:  Isothermal MHD'
  endif
else
  if (adiabatic) then
     write(*,*) 'Physics:  Adiabatic Hydrodynamics'
  else
     write(*,*) 'Physics:  Isothermal Hydrodynamics'
  endif
endif

!
! Allocate and initialize grid variables
!

ihdflast = 0; itablast = 0; ibinlast = 0; ihstlast = 0

!
! User specified number of zones nx1 in namelist - regard that as the
! number of physical zones.  But we need slightly bigger arrays
! due to the presence of ghost zones, so redefine nx1 to include
! the ghost zones.
!
nx1 = nx1+2*NGHOST

grid0%nx1 = nx1
grid0%ibc_x1 = ibc_x1
grid0%obc_x1 = obc_x1
grid0%time = 0.
grid0%x1_0 = x1min
grid0%dx = (x1max-x1min)/(nx1-2*NGHOST)
allocate(grid0%x1(nx1))

!
! Allocate the conserved variables
!
allocate(grid0%d(nx1))
allocate(grid0%m1(nx1))
allocate(grid0%m2(nx1))
allocate(grid0%m3(nx1))

if (mhd) then
  allocate(grid0%b1(nx1))
  allocate(grid0%b2(nx1))
  allocate(grid0%b3(nx1))
  if (adiabatic) then
    allocate(grid0%e(nx1))
    NVAR = 7
  else
    NVAR = 6
  endif
else
  if (adiabatic) then
    allocate(grid0%e(nx1))
    NVAR = 5
  else
    NVAR = 4
  endif
endif

write(*,*) 'NVAR = ',NVAR
!
! call problem setup and grid initialization
!
grid0%time = 0.
nstep = 0

call problem(grid0)
call set_bvals(grid0)
call init_dt(grid0,cour_no)
write(*,*) 'Initial dt = ', grid0%dt

!
! Write out initial data sets
!

nframe = 0

if (dt_hdf .gt. 0.) call hdfout(problem_id,nframe,grid0)
if (dt_tab .gt. 0.) call tabout(problem_id,nframe,grid0)
if (dt_bin .gt. 0.) call binout(problem_id,nframe,grid0)
if (dt_hst .gt. 0.) call hstout(problem_id,nframe,grid0)

!
! initialize timing information
!
call system_clock(COUNT=tstart, COUNT_RATE=clock(2), COUNT_MAX=clock(3))

!
! Physics loop
!

EVOLVE: do WHILE ( (nstep .lt. nlim) .AND. (grid0%time .lt. tlim) )

  if ((tlim-grid0%time).lt. grid0%dt)  grid0%dt = (tlim-grid0%time)

  call integrate_1step(NVAR,grid0,cour_no)
  call set_bvals(grid0)

! write(*,*) 'dt = ', grid0%dt
  nstep = nstep + 1

!
! Data output
!

  if (dt_hdf .gt. 0.) then
    ihdfdump = (grid0%time/dt_hdf)
    if ((ihdfdump-ihdflast).eq.1) then
       nframe = ihdfdump
       call hdfout(problem_id,nframe,grid0)
       ihdflast=ihdfdump
    endif
  endif

  if (dt_tab .gt. 0.) then
    itabdump = (grid0%time/dt_tab)
    if ((itabdump-itablast).eq.1) then
       nframe = itabdump
       call tabout(problem_id,nframe,grid0)
       itablast=itabdump
    endif
  endif
  
  if (dt_bin .gt. 0.) then
    ibindump = (grid0%time/dt_bin)
    if ((ibindump-ibinlast).eq.1) then
       nframe = ibindump
       call binout(problem_id,nframe,grid0)
       ibinlast=ibindump
    endif
  endif

  if (dt_hst .gt. 0.) then
    ihstdump = (grid0%time/dt_hst)
    if ((ihstdump-ihstlast).eq.1) then
       nframe = ihstdump
       call hstout(problem_id,nframe,grid0)
     ihstlast=ihstdump
    endif
  endif

end do EVOLVE

write(*,*) 'Time = ', grid0%time

!
! wrap up program
!
! Timing
! Note:  performance as measured by zone cycles per second is based
! on CPU time elapsed.  
!

call system_clock (COUNT=tstop)
elapsed = float(tstop-tstart)/float(clock(2))
if (elapsed.lt.0.0_r_kind) then
  write(*,*) 'Warning: System clock has cycled. '
  elapsed = float(tstop+clock(3)-tstart)/float(clock(2))
endif
call CPU_TIME(cputime)
write(*,*) 'Elapsed time : ', elapsed,' seconds'
if (cputime .gt. 0.) then
  write(*,*) 'CPU time : ', cputime,' seconds'
  write(*,*) 'Zone-cycles/second: ', float(nstep*grid0%nx1)/cputime
endif

STOP

!
! Error handling
!
901 write(*,*) 'No namelist file, athena.input.  Stopping'
    STOP
902 write(*,*) 'Error reading JOB info in namelist athena.input.  Stopping'
    STOP
903 write(*,*) 'Error reading TIME info in namelist athena.input.  Stopping'
    STOP
904 write(*,*) 'Error reading GRD info in namelist athena.input.  Stopping'
    STOP

END PROGRAM athena
