subroutine set_bvals(agrid)
! 
!  Sets boundary conditions on each edge of the computational volume using
!  integer flags.  The names and supported values of the flags are:
!    ibc_x1 = Inner Boundary Condition for x1
!    obc_x1 = Outer Boundary Condition for x1
! 
!    1 = reflecting; 2 = outflow; 4 = periodic
!
! AUTHORS:
!     James M. Stone, Princeton Univeristy
!     Thomas Gardiner, University of Maryland
!     Peter J. Teuben, University of Maryland
!     John Hawley, University of Virginia
!
!     This software was produced with support from NSF Grant AST-0113571 
!
! Copyright 2003 James M. Stone, Thomas Gardiner,
!                Peter Teuben, John F. Hawley
!
!   This program is free software; you can redistribute it and/or modify
!   it under the terms of the GNU General Public License as published by
!   the Free Software Foundation.
!
!   This program is distributed in the hope that it will be useful,
!   but WITHOUT ANY WARRANTY; without even the implied warranty of
!   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!   GNU General Public License for more details.
!

use athenadefs
IMPLICIT NONE
type (grid) agrid

integer :: i, is, ie

!===========================================================================

is = agrid%is
ie = agrid%ie

SELECT CASE (agrid%ibc_x1)

  CASE (1)  ! reflecting
    do i=1,NGHOST
      agrid%d(is-i)  =  agrid%d(is+i-1) 
      agrid%m1(is-i) = -agrid%m1(is+i-1) 
      agrid%m2(is-i) =  agrid%m2(is+i-1) 
      agrid%m3(is-i) =  agrid%m3(is+i-1) 
      if (ADIABATIC) agrid%e(is-i)  =  agrid%e(is+i-1) 
      if (MHD) then
        agrid%b1(is-i) =  agrid%b1(is+i-1) 
        agrid%b2(is-i) = -agrid%b2(is+i-1) 
        agrid%b3(is-i) = -agrid%b3(is+i-1) 
      endif
    enddo
  CASE (2)  ! outflow
    do i=1,NGHOST
      agrid%d(is-i)  =  agrid%d(is) 
      agrid%m1(is-i) =  MERGE(agrid%m1(is),0._r_kind,(agrid%m1(is) .lt. 0._r_kind)) 
      agrid%m2(is-i) =  agrid%m2(is) 
      agrid%m3(is-i) =  agrid%m3(is) 
      if (ADIABATIC) agrid%e(is-i)  =  agrid%e(is) 
      if (MHD) then
        agrid%b1(is-i) =  agrid%b1(is) 
        agrid%b2(is-i) =  agrid%b2(is) 
        agrid%b3(is-i) =  agrid%b3(is) 
      endif
    enddo

  CASE (4)  ! periodic
    do i=1,NGHOST
      agrid%d(is-i)  =  agrid%d(ie-i+1) 
      agrid%m1(is-i) =  agrid%m1(ie-i+1) 
      agrid%m2(is-i) =  agrid%m2(ie-i+1) 
      agrid%m3(is-i) =  agrid%m3(ie-i+1) 
      if (ADIABATIC) agrid%e(is-i)  =  agrid%e(ie-i+1) 
      if (MHD) then
        agrid%b1(is-i) =  agrid%b1(ie-i+1) 
        agrid%b2(is-i) =  agrid%b2(ie-i+1) 
        agrid%b3(is-i) =  agrid%b3(ie-i+1) 
      endif
    enddo

  CASE DEFAULT
    write(*,*) 'Unknown inner boundary condition flag, ',agrid%ibc_x1,'  Stopping'
    stop

END SELECT

SELECT CASE (agrid%obc_x1)

  CASE (1)  ! reflecting
    do i=1,NGHOST
      agrid%d(ie+i)  =  agrid%d(ie-i+1) 
      agrid%m1(ie+i) = -agrid%m1(ie-i+1) 
      agrid%m2(ie+i) =  agrid%m2(ie-i+1) 
      agrid%m3(ie+i) =  agrid%m3(ie-i+1) 
      if (ADIABATIC) agrid%e(ie+i)  =  agrid%e(ie-i+1) 
      if (MHD) then
        agrid%b1(ie+i) =  agrid%b1(ie-i+1) 
        agrid%b2(ie+i) = -agrid%b2(ie-i+1) 
        agrid%b3(ie+i) = -agrid%b3(ie-i+1) 
      endif
    enddo
  
  CASE (2)  ! outflow
    do i=1,NGHOST
      agrid%d(ie+i)  =  agrid%d(ie) 
      agrid%m1(ie+i) =  MERGE(agrid%m1(ie),0._r_kind,(agrid%m1(ie) .gt. 0._r_kind)) 
      agrid%m2(ie+i) =  agrid%m2(ie) 
      agrid%m3(ie+i) =  agrid%m3(ie) 
      if (ADIABATIC) agrid%e(ie+i)  =  agrid%e(ie) 
      if (MHD) then
        agrid%b1(ie+i) =  agrid%b1(ie) 
        agrid%b2(ie+i) =  agrid%b2(ie) 
        agrid%b3(ie+i) =  agrid%b3(ie) 
      endif
    enddo
  CASE (4)  ! periodic
    do i=1,NGHOST
      agrid%d(ie+i)  =  agrid%d(is+i-1) 
      agrid%m1(ie+i) =  agrid%m1(is+i-1) 
      agrid%m2(ie+i) =  agrid%m2(is+i-1) 
      agrid%m3(ie+i) =  agrid%m3(is+i-1) 
      if (ADIABATIC) agrid%e(ie+i)  =  agrid%e(is+i-1) 
      if (MHD) then
        agrid%b1(ie+i) =  agrid%b1(is+i-1) 
        agrid%b2(ie+i) =  agrid%b2(is+i-1) 
        agrid%b3(ie+i) =  agrid%b3(is+i-1) 
      endif
    enddo

  CASE DEFAULT
    write(*,*) 'Unknown outer boundary condition flag, ',agrid%obc_x1,'  Stopping'
    stop

END SELECT

end subroutine set_bvals
